-----------------------------------------------------------------------------------------------
-- Client Lua Script for Arcompass
-- Author - AlphaArctus (http://wildstar.curseforge.com/members/AlphaArctus)
-- Copyright (c) AlphaArctus. All rights reserved
-----------------------------------------------------------------------------------------------

require "QuestLib"
require "GameLib"
require "ChallengesLib"
require "XmlDoc"
require "PlayerPathLib"

--require "Window"
--require "DialogSys"
--require "Quest"
--require "Tooltip"
--require "Unit"
--require "PublicEvent"
--require "PublicEventObjective"
--require "FriendshipLib"
--require "CraftingLib"

-----------------------------------------------------------------------------------------------
-- Arcompass Module Definition
-----------------------------------------------------------------------------------------------
local Arcompass = {} 

local L = Apollo.GetPackage("Gemini:Locale-1.0").tPackage:GetLocale("Arcompass", true)
local GeminiLocale = Apollo.GetPackage("Gemini:Locale-1.0").tPackage

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local ktInstanceSettingTypeStrings =
{
	Veteran = Apollo.GetString("MiniMap_Veteran"),
	Rallied = Apollo.GetString("MiniMap_Rallied"),
}

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Arcompass:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

    return o
end


function Arcompass:Init()
	Apollo.RegisterAddon(self)
end

function Arcompass:LoadDefaults()
	self.tToggleOptions =
	{
		tLockFrame 			= { name = "LockFrame",		isToggled = false,	opSetting = nil,	opMin = nil,	opMax = nil,	opStep = nil,	buttonStr = "" },
		tZoneText 			= { name = "ZoneText",		isToggled = true,	opSetting = nil,	opMin = nil,	opMax = nil,	opStep = nil,	buttonStr = "" },
		tBackground 		= { name = "Background",	isToggled = true,	opSetting = nil,	opMin = nil,	opMax = nil,	opStep = nil,	buttonStr = "" },
		tCompassBar 		= { name = "CompassBar",	isToggled = true,	opSetting = 100,	opMin = 1,		opMax = 100,	opStep = 10,	buttonStr = "Compass" },
		tPlayerMarker 		= { name = "PlayerMarker",	isToggled = true,	opSetting = 100,	opMin = 1,		opMax = 100,	opStep = 10,	buttonStr = "PC Marker" },
		tOpacity 			= { name = "Opacity",		isToggled = false,	opSetting = nil,	opMin = nil,	opMax = nil,	opStep = nil,	buttonStr = "" },
		tUnitPreview		= { name = "UnitPreview",	isToggled = true,	opSetting = nil,	opMin = nil,	opMax = nil,	opStep = nil,	buttonStr = "" },
		tAutoOpacity		= { name = "AutoOpacity",	isToggled = true,	opSetting = nil,	opMin = nil,	opMax = nil,	opStep = nil,	buttonStr = "" },
		tAutoHide			= { name = "AutoHide",		isToggled = false,	opSetting = nil,	opMin = nil,	opMax = nil,	opStep = nil,	buttonStr = "" },
	}
	self.tMarkerFilters =
	{
		tQuestUnits 		= { name = "QuestUnits",		isToggled = true,	bShowPreview = true,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 150,	opSetting = 150,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Quest NPCs" },
		tQuestTracked 		= { name = "QuestTracked",		isToggled = true,	bShowPreview = false,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 750,	opSetting = 750,	opMin = 350,	opMax = 1500,	opStep = 25,	opBase = 50,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Tracked Quests" },
		tQuestTargets 		= { name = "QuestTargets",		isToggled = true,	bShowPreview = true,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 100,	opSetting = 100,	opMin = 25,		opMax = 350,	opStep = 25,	opBase = 80,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Quest Targets" },
		tLoreDatacube 		= { name = "LoreDatacube",		isToggled = true,	bShowPreview = true,	opAutoMin = 2,	opAutoMax = 6,	opAutoSetting = 350,	opSetting = 350,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 50,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Lore / Datacube" },
		tMailBank 			= { name = "MailBank",			isToggled = true,	bShowPreview = false,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 150,	opSetting = 150,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Mailbox / Bank" },
		tPathMissions 		= { name = "Missions",			isToggled = false,	bShowPreview = false,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 350,	opSetting = 350,	opMin = 150,	opMax = 1500,	opStep = 25,	opBase = 70,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Path Missions" },
		tPathTargets 		= { name = "PathTargets",		isToggled = false,	bShowPreview = false,	opAutoMin = 1,	opAutoMax = 6,	opAutoSetting = 50,		opSetting = 50,		opMin = 25,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Path Targets" },
		tChallenges 		= { name = "Challenges",		isToggled = false,	bShowPreview = false,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 350,	opSetting = 350,	opMin = 150,	opMax = 1500,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Challenges" },
		tChallengeTargets	= { name = "ChallengeTargets",	isToggled = false,	bShowPreview = true,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 50,		opSetting = 50,		opMin = 25,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Challenge Targets" },
		tPublicEvents 		= { name = "PublicEvents",		isToggled = false,	bShowPreview = false,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 150,	opSetting = 150,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Public Events" },
		tVendors 			= { name = "Vendors",			isToggled = false,	bShowPreview = true,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 150,	opSetting = 150,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Vendors" },
		tInstancePortals 	= { name = "InstancePortals",	isToggled = false,	bShowPreview = false,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 150,	opSetting = 150,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Instance Portals" },
		tFlightPaths 		= { name = "FlightPaths",		isToggled = false,	bShowPreview = false,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 150,	opSetting = 150,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Flight Paths" },
		tBindPoints 		= { name = "BindPoints",		isToggled = false,	bShowPreview = false,	opAutoMin = 1,	opAutoMax = 6,	opAutoSetting = 150,	opSetting = 150,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Bind Points" },
		tMiningNodes 		= { name = "MiningNodes",		isToggled = false,	bShowPreview = true,	opAutoMin = 1,	opAutoMax = 6,	opAutoSetting = 250,	opSetting = 250,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Mining Nodes" },
		tRelicNodes 		= { name = "RelicNodes",		isToggled = false,	bShowPreview = true,	opAutoMin = 1,	opAutoMax = 6,	opAutoSetting = 250,	opSetting = 250,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Relic Nodes" },
		tSurvivalistNodes 	= { name = "SurvivalistNodes",	isToggled = false,	bShowPreview = true,	opAutoMin = 1,	opAutoMax = 6,	opAutoSetting = 250,	opSetting = 250,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Survivalist Nodes" },
		tFarmingNodes 		= { name = "FarmingNodes",		isToggled = false,	bShowPreview = true,	opAutoMin = 1,	opAutoMax = 6,	opAutoSetting = 250,	opSetting = 250,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Farming Nodes" },
		tTradeskills 		= { name = "Tradeskills",		isToggled = false,	bShowPreview = false,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 150,	opSetting = 150,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Tradeskill" },
		tNeutral 			= { name = "CreaturesN",		isToggled = false,	bShowPreview = true,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 50,		opSetting = 50,		opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Neutral NPCs" },
		tHostile 			= { name = "CreaturesH",		isToggled = false,	bShowPreview = true,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 50,		opSetting = 50,		opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Hostile NPCs" },
		tTrainer 			= { name = "Trainer",			isToggled = false,	bShowPreview = true,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 150,	opSetting = 150,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Trainers" },
		tFriends 			= { name = "Friends",			isToggled = false,	bShowPreview = true,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 350,	opSetting = 350,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Friends / Guild" },
		tRivals 			= { name = "Rivals",			isToggled = false,	bShowPreview = true,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 150,	opSetting = 150,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Rivals" },
		tWeaponDrop 		= { name = "WeaponDrop",		isToggled = true,	bShowPreview = true,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 300,	opSetting = 300,	opMin = 50,		opMax = 350,	opStep = 25,	opBase = 50,	inRange = 0,	wndClosestObj = nil,	buttonStr = "" },
		tFriendly 			= { name = "CreaturesF",		isToggled = false,	bShowPreview = true,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 50,		opSetting = 50,		opMin = 50,		opMax = 350,	opStep = 25,	opBase = 90,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Neutral NPCs" },
		tGroup 				= { name = "GroupMembers",		isToggled = true,	bShowPreview = false,	opAutoMin = 5,	opAutoMax = 15,	opAutoSetting = 1500,	opSetting = 1500,	opMin = 150,	opMax = 1500,	opStep = 25,	opBase = 50,	inRange = 0,	wndClosestObj = nil,	buttonStr = "Group Members" },
	}
	self.tMarkerSprites =
	{
		ChallengeTargets		= { strSprite = "CRB_TargetFrameRewardPanelSprites:sprTargetFrame_Challenge", tMarkerFilter = self.tMarkerFilters.tChallengeTargets },
		Challenges				= { strSprite = "CRB_ChallengeTrackerSprites:sprChallengeTypeGenericLarge", tMarkerFilter = self.tMarkerFilters.tChallenges },
		WeaponDrop				= { strSprite = "CRB_MinimapSprites:sprMM_ZoneBenefit", tMarkerFilter = self.tMarkerFilters.tWeaponDrop },
		Hostile					= { strSprite = "CRB_NameplateSprites:sprNp_TargetSecondaryHostile", tMarkerFilter = self.tMarkerFilters.tHostile },
		Neutral					= { strSprite = "CRB_NameplateSprites:sprNp_TargetSecondaryNeutral", tMarkerFilter = self.tMarkerFilters.tNeutral },
		MiningNodes				= { strSprite = "IconSprites:Icon_MapNode_Map_Node_Mining", tMarkerFilter = self.tMarkerFilters.tMiningNodes },
		RelicNodes				= { strSprite = "IconSprites:Icon_MapNode_Map_Node_Relic", tMarkerFilter = self.tMarkerFilters.tRelicNodes },
		FarmingNodes			= { strSprite = "IconSprites:Icon_MapNode_Map_Node_Plant", tMarkerFilter = self.tMarkerFilters.tFarmingNodes },
		SurvivalistNodes		= { strSprite = "IconSprites:Icon_MapNode_Map_Node_Tree", tMarkerFilter = self.tMarkerFilters.tSurvivalistNodes },
		Friend					= { strSprite = "IconSprites:Icon_Windows_UI_CRB_Friend", tMarkerFilter = self.tMarkerFilters.tFriends },
		Rival					= { strSprite = "IconSprites:Icon_MapNode_Map_Rival", tMarkerFilter = self.tMarkerFilters.tRivals },
		QuestArrowSmall			= { strSprite = "sprMM_PathArrow", tMarkerFilter = self.tMarkerFilters.tQuestTracked },
		QuestArrow				= { strSprite = "ArcompassSprites:QuestArrow", tMarkerFilter = self.tMarkerFilters.tQuestTracked }, --"sprMM_QuestArrow"
		QuestArrowActive		= { strSprite = "ArcompassSprites:QuestArrowActive", tMarkerFilter = self.tMarkerFilters.tQuestTracked }, -- "sprMM_ActiveQuestArrow"
		QuestTarget				= { strSprite = "sprMM_TargetObjective", tMarkerFilter = self.tMarkerFilters.tQuestTargets },
		QuestCreature			= { strSprite = "sprMM_TargetCreature", tMarkerFilter = self.tMarkerFilters.tQuestTargets },
		QuestReward				= { strSprite = "sprMM_QuestCompleteUntracked", tMarkerFilter = self.tMarkerFilters.tQuestUnits },
		QuestNew				= { strSprite = "IconSprites:Icon_MapNode_Map_Quest", tMarkerFilter = self.tMarkerFilters.tQuestUnits },
		QuestReceiving			= { strSprite = "sprMM_QuestCompleteOngoing", tMarkerFilter = self.tMarkerFilters.tQuestUnits },
		QuestNewTradeskill		= { strSprite = "IconSprites:Icon_MapNode_Map_Quest", tMarkerFilter = self.tMarkerFilters.tQuestUnits },
		TalkTo					= { strSprite = "IconSprites:Icon_MapNode_Map_Chat", tMarkerFilter = self.tMarkerFilters.tQuestUnits },
		Mail					= { strSprite = "IconSprites:Icon_MapNode_Map_Mailbox", tMarkerFilter = self.tMarkerFilters.tMailBank },
		Bank					= { strSprite = "IconSprites:Icon_MapNode_Map_Bank", tMarkerFilter = self.tMarkerFilters.tMailBank },
		GuildBank				= { strSprite = "IconSprites:Icon_MapNode_Map_Bank", tMarkerFilter = self.tMarkerFilters.tMailBank },
		CommodityMarketplace	= { strSprite = "IconSprites:Icon_MapNode_Map_CommoditiesExchange", tMarkerFilter = self.tMarkerFilters.tMailBank },
		ItemAuctionhouse		= { strSprite = "IconSprites:Icon_MapNode_Map_AuctionHouse", tMarkerFilter = self.tMarkerFilters.tMailBank },
		FlightPathSettler		= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Flight", tMarkerFilter = self.tMarkerFilters.tFlightPaths },
		FlightPath				= { strSprite = "IconSprites:Icon_MapNode_Map_Taxi", tMarkerFilter = self.tMarkerFilters.tFlightPaths },
		FlightPathNew			= { strSprite = "IconSprites:Icon_MapNode_Map_Taxi", tMarkerFilter = self.tMarkerFilters.tFlightPaths },
		InstancePortal			= { strSprite = "IconSprites:Icon_MapNode_Map_Portal", tMarkerFilter = self.tMarkerFilters.tInstancePortals },
		BindPoint				= { strSprite = "IconSprites:Icon_MapNode_Map_Gate", tMarkerFilter = self.tMarkerFilters.tBindPoints },
		BindPointCurrent		= { strSprite = "IconSprites:Icon_MapNode_Map_Gate", tMarkerFilter = self.tMarkerFilters.tBindPoints },
		Trainer					= { strSprite = "IconSprites:Icon_MapNode_Map_Trainer", tMarkerFilter = self.tMarkerFilters.tTrainer },
		TradeskillTrainer		= { strSprite = "IconSprites:Icon_MapNode_Map_Trainer", tMarkerFilter = self.tMarkerFilters.tTrainer },
		CraftingStation			= { strSprite = "IconSprites:Icon_MapNode_Map_Tradeskill", tMarkerFilter = self.tMarkerFilters.tTradeskills },
		SettlerImprovement		= { strSprite = "CRB_MinimapSprites:sprMM_SmallIconSettler", tMarkerFilter = self.tMarkerFilters.tPathMissions },
		GroupMember				= { strSprite = "CRB_MinimapSprites:sprMM_Group", tMarkerFilter = self.tMarkerFilters.tGroup },
		Dye						= { strSprite = "IconSprites:Icon_MapNode_Map_DyeSpecialist", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorGeneral			= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorArmor				= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Armor", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorConsumable		= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Consumable", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorElderGem			= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_ElderGem", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorHousing			= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Housing", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorFlight			= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Flight", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorMount				= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Mount", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorRenown			= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Renown", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorReputation		= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Reputation", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorResourceConversion= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_ResourceConversion", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorTradeskill		= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Tradeskill", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorWeapon			= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Weapon", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorPvPArena			= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Prestige_Arena", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorPvPBattlegrounds	= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Prestige_Battlegrounds", tMarkerFilter = self.tMarkerFilters.tVendors },
		VendorPvPWarplots		= { strSprite = "IconSprites:Icon_MapNode_Map_Vendor_Prestige_Warplot", tMarkerFilter = self.tMarkerFilters.tVendors },
		Datacube				= { strSprite = "CRB_MegamapSprites:sprMap_IconCompletion_Lore", tMarkerFilter = self.tMarkerFilters.tLoreDatacube },
		Path_Scientist			= { strSprite = "ArcompassSprites:PathScientist", tMarkerFilter = self.tMarkerFilters.tPathMissions }, -- "IconSprites:Icon_MapNode_Map_Scientist"
		Path_Explorer			= { strSprite = "ArcompassSprites:PathExplorer", tMarkerFilter = self.tMarkerFilters.tPathMissions }, -- "IconSprites:Icon_MapNode_Map_Explorer"
		Path_Settler			= { strSprite = "ArcompassSprites:PathSettler", tMarkerFilter = self.tMarkerFilters.tPathMissions }, -- "IconSprites:Icon_MapNode_Map_Settler"
		Path_Soldier			= { strSprite = "ArcompassSprites:PathSoldier", tMarkerFilter = self.tMarkerFilters.tPathMissions }, -- "IconSprites:Icon_MapNode_Map_Soldier"
		Scientist_Scannable		= { strSprite = "CRB_MinimapSprites:sprMM_SmallIconScientist", tMarkerFilter = self.tMarkerFilters.tPathTargets },
		Explorer				= { strSprite = "CRB_MinimapSprites:sprMM_SmallIconExplorer", tMarkerFilter = self.tMarkerFilters.tPathTargets },
		Scientist_Raw			= { strSprite = "CRB_PlayerPathSprites:spr_Path_Scientist_Stretch", tMarkerFilter = self.tMarkerFilters.tPathTargets },
		SettlerCollect			= { strSprite = "CRB_PlayerPathSprites:spr_Path_Settler_Stretch", tMarkerFilter = self.tMarkerFilters.tPathTargets },
		Friendly				= { strSprite = "CRB_NameplateSprites:sprNp_TargetSecondaryFriendly", tMarkerFilter = self.tMarkerFilters.tFriendly }
		--SchoolOfFishNode		= { strSprite = "IconSprites:Icon_MapNode_Map_Node_Fishing", tMarkerFilter = self.tMarkerFilters.tFishing },
		--QuestNewSoon			= { strSprite = "IconSprites:Icon_MapNode_Map_Quest_Disabled", tMarkerFilter = self.tMarkerFilters.tQuestUnits },
		--PublicEventKill		= "sprMM_TargetCreature",
		--PublicEventTarget		= "sprMM_TargetObjective",
		--PvPExileCarry			= "IconSprites:Icon_MapNode_Map_PvP_ExileCarry",
		--PvPDominionCarry		= "IconSprites:Icon_MapNode_Map_PvP_DominionCarry",
		--PvPNeutralCarry		= "IconSprites:Icon_MapNode_Map_PvP_NeutralCarry",
		--PvPExileCap1			= "IconSprites:Icon_MapNode_Map_PvP_ExileCap",
		--PvPDominionCap1		= "IconSprites:Icon_MapNode_Map_PvP_DominionCap",
		--PvPNeutralCap1		= "IconSprites:Icon_MapNode_Map_PvP_NeutralCap",
		--PvPExileCap2			= "IconSprites:Icon_MapNode_Map_PvP_ExileCap",
		--PvPDominionCap2		= "IconSprites:Icon_MapNode_Map_PvP_DominionCap",
		--PvPNeutralCap2		= "IconSprites:Icon_MapNode_Map_PvP_NeutralCap",
		--PvPBattleAlert		= "IconSprites:Icon_MapNode_Map_PvP_BattleAlert",
	}
	
	self.nUiWidth = 250
	self.eCompassBarTypes = { [1]="CRB_Basekit:kitDivider_Horiz_HoloDotted", [2]="BK3:UI_BK3_Holo_InsetDivider", [3]=""}
	self.nCompassBar = 1
end

-----------------------------------------------------------------------------------------------
-- Arcompass OnLoad
-----------------------------------------------------------------------------------------------
function Arcompass:OnLoad()
	self.AddonVersion = 0.24

	self.xmlDoc = XmlDoc.CreateFromFile("Arcompass.xml")	
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	Apollo.LoadSprites("ArcompassSprites.xml")
	self.playerUnit = nil
	self.playerGuild = nil
	self.arcUnitBacklog = {}
	
	--self.arcQuests = {}
	--self.arcPathMissions = {}

	self.timerMouseOver = nil
	self.bMouseOverTimerActive = false

	self.playerPathMarker = nil
	self.tWorldZone = {}
	self.arcCoords = {
		north = { nRelativePos = 0,	angle = 0,		pId = nil,	pixieInfo = {
					strText = "N",
					strFont = "CRB_Interface11_BBO",
					bLine = false,
					cr = { a=0, r=1.00, g=1.00, b=1.00 },
					crText = { a=1, r=1.00, g=1.00, b=1.00 },
					loc = {
						fPoints = { 0, 0, 0, 0 },
						nOffsets = { }
					},
					flagsText = {
						DT_CENTER = true,
						DT_VCENTER = true
					}
				} },
		east = 	{ nRelativePos = 0,	angle = 90,		pId = nil,	pixieInfo = {
					strText = "E",
					strFont = "CRB_Interface11_BBO",
					bLine = false,
					cr = { a=0, r=1.00, g=1.00, b=1.00 },
					crText = { a=1, r=1.00, g=1.00, b=1.00 },
					loc = {
						fPoints = { 0, 0, 0, 0 },
						nOffsets = { }
					},
					flagsText = {
						DT_CENTER = true,
						DT_VCENTER = true
					}
				} },
		south = { nRelativePos = 0,	angle = 180,	pId = nil,	pixieInfo = {
					strText = "S",
					strFont = "CRB_Interface11_BBO",
					bLine = false,
					cr = { a=0, r=1.00, g=1.00, b=1.00 },
					crText = { a=1, r=1.00, g=1.00, b=1.00 },
					loc = {
						fPoints = { 0, 0, 0, 0 },
						nOffsets = { }
					},
					flagsText = {
						DT_CENTER = true,
						DT_VCENTER = true
					}
				} },
		west = { tnRelativePos = 0,	angle = -90,	pId = nil,	pixieInfo = {
					strText = "W",
					strFont = "CRB_Interface11_BBO",
					bLine = false,
					cr = { a=0, r=1.00, g=1.00, b=1.00 },
					crText = { a=1, r=1.00, g=1.00, b=1.00 },
					loc = {
						fPoints = { 0, 0, 0, 0 },
						nOffsets = { }
					},
					flagsText = {
						DT_CENTER = true,
						DT_VCENTER = true
					}
				} }
	}

	self:LoadDefaults()

	self.bOptionsShown = false
	self.tWindowSettings = {}
	self.bWindowResize = false
	self.bMainWindowShown = true
	self.tPlayerPosition = {}
	self.tMarkerTypes = { Quest=1, Unit=2, Path=3, Challenge=4, Group=5 }
	self.arcMarkers = {}
	self.arcMarkersCollections = {}
	self.arcMarkersDraw = {}
	self.nMousePositionX = nil
	for i, tMarkerFilter in pairs(self.tMarkerFilters) do
		self.arcMarkersCollections[tMarkerFilter.name] = {}
	end
		
	self.bInitLoad = true
	
	Apollo.RegisterSlashCommand("reloadarc", "OnReset", self)

	--Unit Events
	Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
	Apollo.RegisterEventHandler("UnitDestroyed", "OnUnitDestroyed", self)
	Apollo.RegisterEventHandler("UnitActivationTypeChanged", "OnUnitCreated", self)
	
	Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
	
	-- Zone Events
	--Apollo.RegisterEventHandler("VarChange_ZoneName", "OnZoneNameChanged", self)
	Apollo.RegisterEventHandler("SubZoneChanged", "OnZoneNameChanged", self)
	Apollo.RegisterEventHandler("ChangeWorld", "OnChangeWorld", self)
	
	-- Quest Events
	Apollo.RegisterEventHandler("QuestObjectiveUpdated", "OnQuestObjectiveChanged", self)
	Apollo.RegisterEventHandler("QuestStateChanged", "OnQuestStateChanged", self)
	Apollo.RegisterEventHandler("QuestTrackedChanged", "OnQuestStateChanged", self)

	--[[Apollo.RegisterEventHandler("QuestTrackedChanged", "OnQTC", self)
	Apollo.RegisterEventHandler("QuestObjectiveUpdated", "OnQOU", self)
	Apollo.RegisterEventHandler("QuestStateChanged", "OnQSC", self)]]--
	
	-- Path Events
	Apollo.RegisterEventHandler("PlayerPathMissionUpdate", "OnPlayerPathMissionUpdate", self)
	Apollo.RegisterEventHandler("PlayerPathMissionAdvanced", "OnPlayerPathMissionUpdate", self)
	Apollo.RegisterEventHandler("PlayerPathMissionComplete", "OnPlayerPathMissionUpdate", self)
	Apollo.RegisterEventHandler("PlayerPathUpdate", "OnPlayerPathMissionUpdate", self)
	
	-- Group Events
	Apollo.RegisterEventHandler("Group_Join", "OnGroupChanged", self) -- ()
	Apollo.RegisterEventHandler("Group_Add", "OnGroupChanged", self) -- ( name )
	Apollo.RegisterEventHandler("Group_Remove",	"OnGroupChanged", self)	-- ( name, reason )
	Apollo.RegisterEventHandler("Group_Left", "OnGroupChanged", self) -- ( reason )
	Apollo.RegisterEventHandler("Group_UpdatePosition", "OnGroupUpdatePosition", self)--
	
	-- Challenge Events
	Apollo.RegisterEventHandler("ChallengeUpdate", "OnUpdateChallenges", self)
	Apollo.RegisterEventHandler("ChallengeUpdated", "OnUpdateChallenges", self)
	Apollo.RegisterEventHandler("ChallengeUnlocked", "OnUpdateChallenges", self)
	Apollo.RegisterEventHandler("ChallengeFailArea", "OnChallengeInitEnd", self)
	Apollo.RegisterEventHandler("ChallengeFailGeneric", "OnChallengeInitEnd", self)
	Apollo.RegisterEventHandler("ChallengeFailTime", "OnChallengeInitEnd", self)
	Apollo.RegisterEventHandler("ChallengeLeftArea", "OnChallengeInitEnd", self)
	Apollo.RegisterEventHandler("ChallengeActivate", "OnChallengeInitEnd", self)
	Apollo.RegisterEventHandler("ChallengeCompleted", "OnChallengeInitEnd", self)
	Apollo.RegisterEventHandler("ChallengeAbandon", "OnChallengeInitEnd", self)

	--[[Apollo.RegisterEventHandler("ChallengeUpdate", "OnCU", self)
	Apollo.RegisterEventHandler("ChallengeUpdated", "OnCUed", self)
	Apollo.RegisterEventHandler("ChallengeUnlocked", "OnCUn", self)
	Apollo.RegisterEventHandler("ChallengeFailArea", "OnCFA", self)
	Apollo.RegisterEventHandler("ChallengeFailGeneric", "OnCFG", self)
	Apollo.RegisterEventHandler("ChallengeFailTime", "OnCFT", self)
	Apollo.RegisterEventHandler("ChallengeLeftArea", "OnCLA", self)
	Apollo.RegisterEventHandler("ChallengeActivate", "OnCA", self)
	Apollo.RegisterEventHandler("ChallengeCompleted", "OnCC", self)
	Apollo.RegisterEventHandler("ChallengeAbandon", "OnCA", self)]]--

	--Apollo.RegisterEventHandler("VarChange_FrameCount", "OnUpdate", self)
	
	--Apollo.RegisterEventHandler("TargetUnitChanged", "OnTargetUnitChanged", self)

	self.timerDraw = ApolloTimer.Create(0.03, true, "OnDraw", self)
end

--[[
function Arcompass:OnCU()
	Print("ChallengeUpdate")
end

function Arcompass:OnCUed()
	Print("ChallengeUpdated") --When activated, and objectives killed
end

function Arcompass:OnCUn()
	Print("ChallengeUnlocked") --When challenge is first discovered and autostarted
end

function Arcompass:OnCFA()
	Print("ChallengeFailArea")
end

function Arcompass:OnCFG()
	Print("ChallengeFailGeneric")
end

function Arcompass:OnCFT()
	Print("ChallengeFailTime")
end

function Arcompass:OnCLA()
	Print("ChallengeLeftArea")
end

function Arcompass:OnCA()
	Print("ChallengeActivate")
end

function Arcompass:OnCC()
	Print("ChallengeCompleted")
end

function Arcompass:OnCA()
	Print("ChallengeAbandon") --When abandoned, and before activating?? wth
end

function Arcompass:OnQTC()
	Print("Quest Tracked Changed") --When tracked/untracked: Update all units to just those tracked?
end

function Arcompass:OnQOU()
	Print("Quest Objective Updated") --When objectives are completed: Update all tracked units
end

function Arcompass:OnQSC()
	Print("Quest State Changed") --When quests completed/started: Update all quests and all units
end]]--

function Arcompass:OnTargetUnitChanged(unit) --Personal beta testing function
	if unit == nil then return end
	Print(unit:GetId())
	--self:OnGroupChanged()
	--self:RefreshMarker(self.arcMarkers["unit"..tostring(unit:GetId())])
	--self:UpdateChallenges()
	--self:OnQuestStateChanged()
	--self:OnPlayerPathMissionUpdate()
end

function Arcompass:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Arcompass", {"ToggleOptionsWindow", "", "CRB_MinimapSprites:sprMM_ZoneBenefit"})
end

function Arcompass:OnDocLoaded()
	if self.xmlDoc:IsLoaded() then
		self.wndMain = Apollo.LoadForm(self.xmlDoc, "Arcompass", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return Apollo.AddonLoadStatus.LoadingError
		end
		self.wndCoords = self.wndMain:FindChild("Coords")
		self.wndMain:SetSizingMinimum(250, 20)
		self.wndMain:SetSizingMaximum(2560, 20)
		
		self.wndQuest = self.wndMain:FindChild("QuestWindow")
		self.wndUnit = self.wndMain:FindChild("UnitWindow")
		
		-- Load Closest Markers Windows
		for i, tMarkerFilter in pairs(self.tMarkerFilters) do
			if tMarkerFilter.name == self.tMarkerFilters.tQuestTracked.name or
				tMarkerFilter.name == self.tMarkerFilters.tPathMissions.name or 
				tMarkerFilter.name == self.tMarkerFilters.tChallenges.name then
				tMarkerFilter.wndClosestObj = Apollo.LoadForm(self.xmlDoc , "ArcompassClosestObj", self.wndQuest, self)
			else
				tMarkerFilter.wndClosestObj = Apollo.LoadForm(self.xmlDoc , "ArcompassClosestObj", self.wndUnit, self)
			end
			tMarkerFilter.wndClosestObj:Show(false)
		end

		-- Load Tooltip for Quest/Path/Challenges
		self.wndTooltip = Apollo.LoadForm(self.xmlDoc , "ArcompassTooltip", nil, self)
		self.wndTooltipPixie = {
			loc = { 
				fPoints = { 0, 0, 0, 0 },
				nOffsets = { 47, 39, 305, 59 }
			},
			cr = { a=0.00, r=1.00, g=1.00, b=1.00 },
			crText = { a=1, r=0.10, g=1.00, b=1.00 },
			fRotation = 0, 
			strFont = "CRB_InterfaceSmall",
			strText = "",
			flagsText = {
				DT_VCENTER = true,
				DT_CENTER = true
			}
		}
		self.wndTooltipPixieId = self.wndTooltip:AddPixie(self.wndTooltipPixie)
		self.wndTooltip:Show(false)
		self.wndArcompassOptions = Apollo.LoadForm(self.xmlDoc , "ArcompassOptions", nil, self)
		self.wndArcompassOptions:Show(false)

		-- Load Tooltip for everything else
		self.wndTooltipAlt = Apollo.LoadForm(self.xmlDoc , "ArcompassClosestObj", nil, self)
		self.wndTooltipAlt:SetStyle("CloseOnExternalClick", true)
		self.wndTooltipAlt:Show(false)
		self.wndTooltipAltText = Apollo.LoadForm(self.xmlDoc , "ArcompassClosestObj", nil, self)
		self.wndTooltipAltText:SetStyle("CloseOnExternalClick", true)
		self.wndTooltipAltText:Show(false)
		self.wndTooltipAltText:FindChild("WindowTarget"):Show(false)

		-- Set Options
		-- Lock Frame
		local btnLockFrame = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tLockFrame.name)
		btnLockFrame:SetData(self.tToggleOptions.tLockFrame)
		local bLockFrame = self.tToggleOptions.tLockFrame.isToggled
		btnLockFrame:SetCheck(bLockFrame)
		self.wndMain:SetStyle("Moveable", not bLockFrame)
		self.wndMain:SetStyle("Sizable", not bLockFrame)
		self.wndMain:FindChild("ResizeLeft"):Show(false) --not bLockFrame)
		self.wndMain:FindChild("ResizeRight"):Show(false) --not bLockFrame)
		btnLockFrame:SetTooltip("Lock Frame "..(bLockFrame and "[LOCKED]" or "[UNLOCKED]"))
		
		-- Zone Text
		local btnZoneText = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tZoneText.name)
		btnZoneText:SetData(self.tToggleOptions.tZoneText)
		local bZoneText = self.tToggleOptions.tZoneText.isToggled
		btnZoneText:SetCheck(bZoneText)
		btnZoneText:SetTooltip("Show Zone Info "..(bZoneText and "[ON]" or "[OFF]"))
		self.wndZoneText = self.wndMain:FindChild("ZoneText")
		self.wndZoneText:Show(bZoneText)
		self.wndPlayerCoord = self.wndMain:FindChild("PlayerCoord")
		self.wndPlayerCoord:Show(bZoneText)
		
		-- Background
		local btnBackground = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tBackground.name)
		btnBackground:SetData(self.tToggleOptions.tBackground)
		local bBackground = self.tToggleOptions.tBackground.isToggled
		btnBackground:SetCheck(bBackground)
		btnBackground:SetTooltip("Show Compass BG "..(bBackground and "[ON]" or "[OFF]"))
		self.wndMain:FindChild("ArcompassBackground"):Show(bBackground)
		
		-- Compass Bar
		local btnCompassBar = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tCompassBar.name)
		btnCompassBar:SetData(self.tToggleOptions.tCompassBar)
		btnCompassBar:SetTooltip("Change Compass Bar Type")
		local wndCompassBackground = self.wndMain:FindChild("CompassBackground")
		wndCompassBackground:SetSprite(self.eCompassBarTypes[self.nCompassBar])
		wndCompassBackground:SetOpacity(self.tToggleOptions.tCompassBar.opSetting*0.01)

		-- Player Marker
		local btnPlayerMarker = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tPlayerMarker.name)
		btnPlayerMarker:SetData(self.tToggleOptions.tPlayerMarker)
		local bPlayerMarker = self.tToggleOptions.tPlayerMarker.isToggled
		btnPlayerMarker:SetCheck(bPlayerMarker)
		btnPlayerMarker:SetTooltip("Show Player Marker "..(bPlayerMarker and "[ON]" or "[OFF]"))
		self.wndPlayerMarker = self.wndMain:FindChild("PlayerMarker")
		self.wndPlayerMarker:Show(bPlayerMarker)
		self.wndPlayerMarker:SetOpacity(self.tToggleOptions.tPlayerMarker.opSetting*0.01)
		
		-- Unit Preview Marker
		local btnUnitPreview = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tUnitPreview.name)
		btnUnitPreview:SetData(self.tToggleOptions.tUnitPreview)
		local bUnitPreview = self.tToggleOptions.tUnitPreview.isToggled
		btnUnitPreview:SetCheck(bUnitPreview)
		btnUnitPreview:SetTooltip("Show Unit Portraits "..(bUnitPreview and "[ON]" or "[OFF]"))

		-- Auto Opacity
		local btnAutoOpacity = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tAutoOpacity.name)
		btnAutoOpacity:SetData(self.tToggleOptions.tAutoOpacity)
		local bAutoOpacity = self.tToggleOptions.tAutoOpacity.isToggled
		btnAutoOpacity:SetCheck(bAutoOpacity)
		btnAutoOpacity:SetTooltip("Auto Opacity Feature "..(bAutoOpacity and "[ON]" or "[OFF]"))

		-- Auto Hide
		local btnAutoHide = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tAutoHide.name)
		btnAutoHide:SetData(self.tToggleOptions.tAutoHide)
		local bAutoHide = self.tToggleOptions.tAutoHide.isToggled
		btnAutoHide:SetCheck(bAutoHide)
		btnAutoHide:SetTooltip("Auto Hide Feature "..(bAutoHide and "[ON]" or "[OFF]"))
		
		-- Opacity Button
		local btnOpacity = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tOpacity.name)
		btnOpacity:SetData(self.tToggleOptions.tOpacity)
		local bOpacity = self.tToggleOptions.tOpacity.isToggled
		btnOpacity:SetCheck(bOpacity)
		btnOpacity:SetTooltip("Show Opacity Sliders "..(bOpacity and "[ON]" or "[OFF]"))
		for i, tMarkerFilter in pairs(self.tMarkerFilters) do
			if tMarkerFilter.name ~= self.tMarkerFilters.tWeaponDrop.name and 
				tMarkerFilter.name ~= self.tMarkerFilters.tFriendly.name then
				-- Set Filter Button
				local wndToggleBtn = self.wndArcompassOptions:FindChild("OptionsBtn"..tMarkerFilter.name)
				wndToggleBtn:SetData(tMarkerFilter)
				wndToggleBtn:SetCheck(tMarkerFilter.isToggled)
				-- Set Filter Background
				local wndSliderBackground = wndToggleBtn:FindChild("SliderBG")
				wndSliderBackground:SetText(tostring(math.floor(tMarkerFilter.opSetting)).."m")
				wndSliderBackground:SetStyle("IgnoreMouse", bOpacity)
				wndSliderBackground:Show(bOpacity)
				if bOpacity then
					wndToggleBtn:SetText("")
				else
					wndToggleBtn:SetText("          "..tMarkerFilter.buttonStr)
				end
				-- Set Filter Slider
				local sliderBar = wndToggleBtn:FindChild("SliderBar")
				sliderBar:SetData(tMarkerFilter)
				sliderBar:SetMinMax(tMarkerFilter.opMin, tMarkerFilter.opMax, tMarkerFilter.opStep)
				sliderBar:SetValue(tMarkerFilter.opSetting)
				sliderBar:SetOpacity(0.75)
			end
		end
		-- Set Compass Bar and Player Marker sliders
		for i, tToggleOption in pairs(self.tToggleOptions) do
			if tToggleOption.opSetting ~= nil then
				-- Set Filter Background
				local wndToggleBtn = self.wndArcompassOptions:FindChild("OptionsBtn"..tToggleOption.name)
				local wndSliderBackground = wndToggleBtn:FindChild("SliderBG")
				wndSliderBackground:SetStyle("IgnoreMouse", bOpacity)
				wndSliderBackground:Show(bOpacity)
				if bOpacity then
					wndToggleBtn:SetText("")
				else
					wndToggleBtn:SetText("          "..tToggleOption.buttonStr)
				end
				-- Set Filter Slider
				local sliderBar = wndToggleBtn:FindChild("SliderBar")
				sliderBar:SetData(tToggleOption)
				sliderBar:SetMinMax(tToggleOption.opMin, tToggleOption.opMax, tToggleOption.opStep)
				sliderBar:SetValue(tToggleOption.opSetting)
				sliderBar:SetOpacity(0.75)
			end
		end
		
		-- Center Button
		self.wndArcompassOptions:FindChild("OptionsBtnCenterBar"):SetTooltip("Centre Compass Bar")
		
		-- Set main window location
		if self.tWindowSettings.nOL then
			local tAnchorOffsets = {}
			tAnchorOffsets.nOL, tAnchorOffsets.nOT, tAnchorOffsets.nOR, tAnchorOffsets.nOB = self.wndMain:GetAnchorOffsets()
			if (self.tWindowSettings.nOL > self.tWindowSettings.nOR) then --If the UI is inversed...
				self.tWindowSettings.nOL = tAnchorOffsets.nOL
				self.tWindowSettings.nOR = tAnchorOffsets.nOR
			end
			self.wndMain:SetAnchorOffsets(self.tWindowSettings.nOL, self.tWindowSettings.nOT, self.tWindowSettings.nOR, self.tWindowSettings.nOT+20)
		end
		self.wndMain:Show(true)
		
		self.timerUpdate = ApolloTimer.Create(0.1, true, "OnUpdate", self)
		self.timerQuestOrder = ApolloTimer.Create(0.2, true, "OnQuestOrder", self)
		
		Apollo.RegisterEventHandler("ToggleOptionsWindow", "ShowOptionsWindow", self)
	end
end


-----------------------------------------------------------------------------------------------
-- Arcompass OnSave/OnRestore EventHandler
-----------------------------------------------------------------------------------------------
function Arcompass:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	local tSavedData = self.tWindowSettings
	tSavedData.Pos = {}
	tSavedData.Pos.nOL, tSavedData.Pos.nOT, tSavedData.Pos.nOR, tSavedData.Pos.nOB = self.wndMain:GetAnchorOffsets()
	tSavedData.AddonVersion = self.AddonVersion
	tSavedData.tMarkerFilters = self.tMarkerFilters
	tSavedData.tToggleOptions = self.tToggleOptions
	tSavedData.nCompassBar = self.nCompassBar
	return tSavedData
end


function Arcompass:OnRestore(eType, tSavedData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	if tSavedData == nil then return end
	
	--if tSavedData.AddonVersion == self.AddonVersion then
	if tSavedData.tMarkerFilters ~= nil then
		for keyname, tMarkerFilter in pairs(tSavedData.tMarkerFilters) do
			if self.tMarkerFilters[keyname] ~= nil then
				self.tMarkerFilters[keyname].isToggled = tMarkerFilter.isToggled
				self.tMarkerFilters[keyname].opSetting = tMarkerFilter.opSetting
			end
		end
	end
	
	if tSavedData.tToggleOptions ~= nil then
		for keyname, tToggleOption in pairs(tSavedData.tToggleOptions) do
			if self.tToggleOptions[keyname] ~= nil then
				self.tToggleOptions[keyname].isToggled = tToggleOption.isToggled
				self.tToggleOptions[keyname].opSetting = tToggleOption.opSetting
			end
		end
	end

	self.nCompassBar = tSavedData.nCompassBar
	--end

	if tSavedData.Pos ~= nil then
		for idx, item in pairs(tSavedData.Pos) do
			self.tWindowSettings[idx] = item
		end	
	end
end


function Arcompass:OnReset()
	self:LoadDefaults()
	RequestReloadUI()
end

-----------------------------------------------------------------------------------------------
-- Arcompass Zone Change/Change World EventHandlers
-----------------------------------------------------------------------------------------------
function Arcompass:OnZoneNameChanged(oVar, strNewZone)
	--Print(strNewZone)
	-- Update Zone Name
	self:UpdateZoneName(GetCurrentZoneName())
	
	-- Reset Quest Markers
	--self:OnQuestStateChanged()
	
	-- Reset Mission Markers
	--self:OnPlayerPathMissionUpdate()
	
	-- Reset Challenges
	--self:OnUpdateChallenges()
end


function Arcompass:OnChangeWorld()
	-- Update Zone Name
	self:UpdateZoneName(GetCurrentZoneName())
	
	-- Reset Quest Markers
	self:OnQuestStateChanged()
	
	-- Reset Mission Markers
	self:OnPlayerPathMissionUpdate()
	
	-- Reset Challenges
	self:OnUpdateChallenges()
	
	self:OnGroupChanged()
end


-----------------------------------------------------------------------------------------------
-- General Events/Functions
-----------------------------------------------------------------------------------------------
function Arcompass:UpdateZoneName(strZoneName)
	if strZoneName == nil then return end

	local tInstanceSettingsInfo = GameLib.GetInstanceSettings()

	local strDifficulty = nil
	if tInstanceSettingsInfo.eWorldDifficulty == GroupLib.Difficulty.Veteran then
		strDifficulty = ktInstanceSettingTypeStrings.Veteran
	end

	local strScaled = nil
	if tInstanceSettingsInfo.bWorldForcesLevelScaling == true then
		strScaled = ktInstanceSettingTypeStrings.Rallied
	end

	local strAdjustedZoneName = strZoneName
	if strDifficulty and strScaled then
		strAdjustedZoneName = strZoneName .. " (" .. strDifficulty .. "-" .. strScaled .. ")"
	elseif strDifficulty then
		strAdjustedZoneName = strZoneName .. " (" .. strDifficulty .. ")"
	elseif strScaled then
		strAdjustedZoneName = strZoneName .. " (" .. strScaled .. ")"
	end

	self.wndZoneText:SetText(strAdjustedZoneName)
end


function Arcompass:OnCharacterCreated()
	if self.playerUnit == nil then return end

	self.unitPlayerDisposition = GameLib.GetPlayerUnit()
	local ePath = self.unitPlayerDisposition:GetPlayerPathType()
	
	if ePath == PlayerPathLib.PlayerPathType_Soldier then
		self.wndArcompassOptions:FindChild("Image_Soldier"):Show(true)
		self.wndArcompassOptions:FindChild("Image_Soldier_1"):Show(true)
		self.playerPathMarker = self.tMarkerSprites.Path_Soldier
	elseif ePath == PlayerPathLib.PlayerPathType_Explorer then
		self.wndArcompassOptions:FindChild("Image_Explorer"):Show(true)
		self.wndArcompassOptions:FindChild("Image_Explorer_1"):Show(true)
		self.playerPathMarker = self.tMarkerSprites.Path_Explorer
	elseif ePath == PlayerPathLib.PlayerPathType_Scientist then
		self.wndArcompassOptions:FindChild("Image_Scientist"):Show(true)
		self.wndArcompassOptions:FindChild("Image_Scientist_1"):Show(true)
		self.playerPathMarker = self.tMarkerSprites.Path_Scientist
	elseif ePath == PlayerPathLib.PlayerPathType_Settler then
		self.wndArcompassOptions:FindChild("Image_Settler"):Show(true)
		self.wndArcompassOptions:FindChild("Image_Settler_1"):Show(true)
		self.playerPathMarker = self.tMarkerSprites.Path_Settler
	end
	
	self:UpdateZoneName(GetCurrentZoneName())
	self:OnQuestStateChanged()
	self:OnPlayerPathMissionUpdate()
	self:OnInterfaceMenuListHasLoaded()
	self:OnUpdateChallenges()
	self:OnGroupChanged()
end


-----------------------------------------------------------------------------------------------
-- Arcompass OnUpdate/OnDraw Local Functions
-----------------------------------------------------------------------------------------------
local function CalculateDistance2D(tPos1, tPos2)
	if not tPos1 or not tPos2 then
		return
	end
	local nDeltaX = tPos2.x - tPos1.x
	local nDeltaZ = tPos2.z - tPos1.z
	local nDistance = math.sqrt(math.pow(nDeltaX, 2) + math.pow(nDeltaZ, 2))
	return nDistance
end

local function CalculateDistance3D(tPos1, tPos2)
	if not tPos1 or not tPos2 then
		return
	end
	local v3new = Vector3.New
	local v3Pos1 = v3new(tPos1)
	local v3Pos2 = v3new(tPos2)
	local v3Diff = v3Pos2 - v3Pos1
	local nLength = v3Diff:Length()
	if nLength ~= nil then return nLength end
end

local function CalculateAngleDiff(a1, a2)
	local ret = a1-a2
	if ret > 180 then
		ret = ret - 360
	elseif ret < -180 then
		ret = ret + 360
	end
	return ret
end

local function CalculatePointFromCentre(width, angle)
	local ret = width
	if angle == 0 then
		return ret
	end
	ret = ret + ((ret / 180)*angle) - (width/2)
	return math.floor(ret)
end

local function CalculateOpacity(indicatorDistance, tMarkerFilter, self)
	if self.tToggleOptions.tAutoOpacity.isToggled == true then
		return (-(tMarkerFilter.opBase*((indicatorDistance^3)/((indicatorDistance^3)+((tMarkerFilter.opAutoSetting)^3))))+100)*0.01
	else
		return (-(tMarkerFilter.opBase*((indicatorDistance^3)/((indicatorDistance^3)+((tMarkerFilter.opSetting)^3))))+100)*0.01
	end
end

function Arcompass:OnQuestOrder()
	local function compare(a, b)
		return a.nDistance < b.nDistance
	end

	------------------------------------------ Sort Quests ------------------------------------------
	local tTemp = {}
	local arcMarkersQuests = self.arcMarkersCollections[self.tMarkerFilters.tQuestTracked.name]
	if arcMarkersQuests ~= nil then
		for k, tMarker in pairs(arcMarkersQuests) do
			if tMarker.nDistance ~= nil then
				local tQuest = { nDistance = tMarker.nDistance, arcMarker = tMarker }
				table.insert(tTemp, tQuest)
			end
		end
		
		table.sort(tTemp, compare)
		
		local nCount = 1
		for k, tMarker in pairs(tTemp) do
			tMarker.arcMarker.tPixie.strText = tostring(nCount)
			nCount = nCount+1
		end
	end
	------------------------------------------ End Quests ------------------------------------------
end

-----------------------------------------------------------------------------------------------
-- Arcompass OnUpdate Timer Functions
-----------------------------------------------------------------------------------------------
function Arcompass:OnUpdate()
	if self.playerUnit == nil or not self.playerUnit:IsValid() then
		if GameLib.GetPlayerUnit() == nil then return end
		self.playerUnit = GameLib.GetPlayerUnit()
		self.playerGuild = GameLib.GetPlayerUnit():GetGuildName()
		self:OnCharacterCreated()
	else
		local tPlayerPosition = self.playerUnit:GetPosition()
		if tPlayerPosition == nil then return end
		local m_floor = math.floor
		
		if self.playerUnit:IsInVehicle() == true then
			local uVehicle = self.playerUnit:GetVehicle()
			if uVehicle ~= nil then
				local tVehiclePosition = uVehicle:GetPosition()
				if tVehiclePosition ~= nil then
					tPlayerPosition = tVehiclePosition
				end
			end
		end

		------------------------------------------ Check backlog ------------------------------------------
		for i, unit in pairs(self.arcUnitBacklog) do
			if unit:IsValid() then
				self:OnUnitCreated(self.arcUnitBacklog[i])
			end
			self.arcUnitBacklog[i] = nil
		end 
		self.arcUnitBacklog = {}
		------------------------------------------ End backlog ------------------------------------------
		
		-- Update player marker position
		if self.bWindowResize then
			self.nUiWidth = self.wndMain:FindChild("QuestWindow"):GetWidth()
			self.wndPlayerMarker:SetAnchorOffsets((self.nUiWidth/2)-16, 8, (self.nUiWidth/2)+16, 40)
			self.bWindowResize = false
		end
		
		local fCalculateDistance3D = CalculateDistance3D
		
		------------------------------------------ Update Markers ------------------------------------------
		for iFilter, tMarkerFilter in pairs(self.tMarkerFilters) do
			if tMarkerFilter.isToggled then
				local nClosestOverallMarker = nil
				local nClosestOverallDistance = 99999
				local nVisibleCount = 0
				
				for i, arcMarker in pairs(self.arcMarkersCollections[tMarkerFilter.name]) do
					local nClosestDistance = 99999
					local tClosestLocation = nil
					
					-- Get closest distance from known locations
					for j, tMarker in pairs(arcMarker.tMarkers) do
						-- Update marker position if it is a unit
						if arcMarker.eType == self.tMarkerTypes.Unit then
							local tLocationRef = arcMarker.uOwner:GetPosition()
							if tLocationRef ~= nil then
								local tLocation = {} 
								tLocation.x = tLocationRef.x
								tLocation.y = tLocationRef.y
								tLocation.z = tLocationRef.z
								tMarker.tLocation = tLocation
							else
								tMarker.tLocation = nil
							end
						elseif arcMarker.eType == self.tMarkerTypes.Group then
							local uGroupMember = GroupLib.GetUnitForGroupMember(arcMarker.nGroupNumber)
							if uGroupMember ~= nil then
								local tLocationRef = uGroupMember:GetPosition()
								if tLocationRef ~= nil then
									local tLocation = {} 
									tLocation.x = tLocationRef.x
									tLocation.y = tLocationRef.y
									tLocation.z = tLocationRef.z
									tMarker.tLocation = tLocation
								end
							end
						end
						
						-- Get unit distance from player
						local nUnitDistance = fCalculateDistance3D(tPlayerPosition, tMarker.tLocation)
						if nUnitDistance == nil then nUnitDistance = 99999 end
						if nUnitDistance < nClosestDistance then
							nClosestDistance = nUnitDistance
							arcMarker.tClosestMarker = tMarker
						end
					end
					
					local bUnitZoneMinimap = true
					if arcMarker.eType == self.tMarkerTypes.Unit then
						if not arcMarker.uOwner:IsVisibleOnCurrentZoneMinimap() then 
							bUnitZoneMinimap = false
						end
					end
					
					-- Set relative compass position and marker settings
					if nClosestDistance ~= 99999 and arcMarker.tClosestMarker ~= nil and bUnitZoneMinimap then
						arcMarker.nDistance = nClosestDistance
						local opacity = 1

						-- Set sprite opacity and sprite string
						local strSprite = nil
						if arcMarker.tMarkerSpriteDisposition ~= nil then
							if arcMarker.tMarkerSpriteDisposition.tMarkerFilter.isToggled then
								opacity = CalculateOpacity(arcMarker.nDistance, arcMarker.tMarkerSpriteDisposition.tMarkerFilter, self)
								strSprite = arcMarker.tMarkerSpriteDisposition.strSprite
							end
						end
						if arcMarker.tMarkerSpriteFilter ~= nil then
							if arcMarker.tMarkerSpriteFilter.tMarkerFilter.isToggled then
								opacity = CalculateOpacity(arcMarker.nDistance, arcMarker.tMarkerSpriteFilter.tMarkerFilter, self)
								strSprite = arcMarker.tMarkerSpriteFilter.strSprite
							end
						end
						if arcMarker.tMarkerSpriteChallenge ~= nil then
							if arcMarker.tMarkerSpriteChallenge.tMarkerFilter.isToggled then
								opacity = CalculateOpacity(arcMarker.nDistance, arcMarker.tMarkerSpriteChallenge.tMarkerFilter, self)
								strSprite = arcMarker.tMarkerSpriteChallenge.strSprite
							end
						end
						
						if opacity > 0.3 and strSprite ~= nil then
							if arcMarker.bHighlighted == true then
								arcMarker.tPixie.cr = { a=1.00, r=1.00, g=1.00, b=1.00 }
							else
								arcMarker.tPixie.cr = { a=opacity, r=1.00, g=1.00, b=1.00 }
							end
							
							if arcMarker.eType == self.tMarkerTypes.Quest then
								if arcMarker.uOwner:IsActiveQuest() then
									arcMarker.tPixie.strSprite = self.tMarkerSprites.QuestArrowActive.strSprite
								else
									arcMarker.tPixie.strSprite = self.tMarkerSprites.QuestArrow.strSprite
								end
							else
								arcMarker.tPixie.strSprite = strSprite
							end
							
							if self.arcMarkersDraw[arcMarker.idMarker] == nil then
								self.arcMarkersDraw[arcMarker.idMarker] = arcMarker
							end

							-- Auto opacity counter
							nVisibleCount = nVisibleCount + 1

							-- Set overall closest marker
							if arcMarker.nDistance < nClosestOverallDistance then
								nClosestOverallDistance = arcMarker.nDistance
								nClosestOverallMarker = arcMarker.idMarker
							end
						else
							if self.arcMarkersDraw[arcMarker.idMarker] ~= nil then
								self:ClearMarker(arcMarker)
							end
						end
					else
						if self.arcMarkersDraw[arcMarker.idMarker] ~= nil then
							self:ClearMarker(arcMarker)
						end
					end
				end
								
				if nClosestOverallMarker ~= nil then
					local arcMarkerClosest = self.arcMarkers[nClosestOverallMarker]
					tMarkerFilter.wndClosestObj:SetData(arcMarkerClosest)
				else
					tMarkerFilter.wndClosestObj:SetData(nil)
				end

				-- Auto opacity update
				if self.tToggleOptions.tAutoOpacity.isToggled then
					if nVisibleCount > tMarkerFilter.opAutoMax then
						if tMarkerFilter.opAutoSetting > tMarkerFilter.opMin then
							tMarkerFilter.opAutoSetting = tMarkerFilter.opAutoSetting - 5
							if tMarkerFilter.opAutoSetting < tMarkerFilter.opMin then
								tMarkerFilter.opAutoSetting = tMarkerFilter.opMin
							end
						end
					elseif nVisibleCount < tMarkerFilter.opAutoMin then
						if tMarkerFilter.opAutoSetting < tMarkerFilter.opMax then
							tMarkerFilter.opAutoSetting = tMarkerFilter.opAutoSetting + 5
							if tMarkerFilter.opAutoSetting > tMarkerFilter.opMax then
								tMarkerFilter.opAutoSetting = tMarkerFilter.opMax
							end
						end
					end
				end
			else
				tMarkerFilter.wndClosestObj:SetData(nil)
			end
		end
		------------------------------------------ End Markers ------------------------------------------
				
		local tPlayerLocation = self.playerUnit:GetPosition()
		local tString = {"(",tostring(math.floor(tPlayerLocation.x)),",",tostring(math.floor(tPlayerLocation.z)),")"}
		self.wndPlayerCoord:SetText(table.concat(tString))		
	end
end


-----------------------------------------------------------------------------------------------
-- Arcompass OnDraw Timer Function
-----------------------------------------------------------------------------------------------
function Arcompass:OnDraw()
	local playerUnit = self.playerUnit
	if playerUnit ~= nil and playerUnit:IsValid() then
	
		-- Get player facing coordinates
		local tPlayerPosition = playerUnit:GetPosition()
		local tPlayerFacing = playerUnit:GetFacing()
		local nUiWidth = self.nUiWidth
		if tPlayerFacing == nil or tPlayerPosition == nil then return end
		
		if self.playerUnit:IsInVehicle() == true then
			local uVehicle = self.playerUnit:GetVehicle()
			if uVehicle ~= nil then
				local tVehiclePosition = uVehicle:GetPosition()
				local tVehicleFacing = uVehicle:GetFacing()
				if tVehiclePosition ~= nil and tVehicleFacing ~= nil then
					tPlayerPosition = tVehiclePosition
					tPlayerFacing = tVehicleFacing
				end
			end
		end

		local m_pi = math.pi
		local m_atan2 = math.atan2
		local m_floor = math.floor
		
		local v_new = Vector3.New
		
		-- Get facing angle
		local nFacingAngle = m_atan2(tPlayerFacing.x, -(tPlayerFacing.z)) * (180/m_pi)
		if nFacingAngle == nil then return end
		
		------------------------------------------ Draw Coordinates ------------------------------------------
		for k, coord in pairs(self.arcCoords) do
			local relativeAngle = CalculateAngleDiff(coord.angle, nFacingAngle) -- Angle difference to the player
			local relativePos = CalculatePointFromCentre(nUiWidth, relativeAngle) -- Position on the compass'
			if relativePos ~= nil then
				-- Build new NESW data
				coord.pixieInfo.loc.nOffsets = { relativePos-8, 10, relativePos+8, 30 }
				if coord.pId ~= nil then
					self.wndCoords:UpdatePixie(coord.pId, coord.pixieInfo)
				else
					coord.pId = self.wndCoords:AddPixie(coord.pixieInfo)
				end
			end
		end
		------------------------------------------ End Coordinates ------------------------------------------
		
		------------------------------------------ Draw Markers ------------------------------------------	
		local MarkerTypes = self.tMarkerTypes
		if self.arcMarkersDraw ~= nil then
			for i, arcMarker in pairs(self.arcMarkersDraw) do
				if arcMarker.tClosestMarker ~= nil then
					local tClosestLocation = arcMarker.tClosestMarker.tLocation
					if tClosestLocation ~= nil then
						local nLocationX = tClosestLocation.x
						local nLocationY = tClosestLocation.y
						local nLocationZ = tClosestLocation.z
						if nLocationX ~= nil and nLocationY ~= nil and nLocationZ ~= nil then
							-- Get relative compass position
							local nUnitAngle = m_atan2(nLocationX-tPlayerPosition.x, -(nLocationZ-tPlayerPosition.z)) * (180/m_pi) -- Angle of the unit relative to 0/North
							local nRelativeAngle = CalculateAngleDiff(nUnitAngle, nFacingAngle) -- Angle difference to the player
							local nRelativePos = CalculatePointFromCentre(nUiWidth, nRelativeAngle) -- Position on the compass
							if nRelativePos < 29 then
								nRelativePos = 29
							elseif nRelativePos > nUiWidth-29 then
								nRelativePos = nUiWidth-29
							end
							arcMarker.nRelativePos = nRelativePos
							
							-- Update pixie position
							local nMarkerWidth = arcMarker.nWidth
							local nMarkerHeight = arcMarker.nHeight
							arcMarker.tPixie.loc.nOffsets = { nRelativePos-(nMarkerWidth/2), 20-(nMarkerHeight/2), nRelativePos+(nMarkerWidth/2), 20+(nMarkerHeight/2) }
							if arcMarker.nPixieId ~= nil then
								arcMarker.wndOwner:UpdatePixie(arcMarker.nPixieId, arcMarker.tPixie)
							else
								arcMarker.nPixieId = arcMarker.wndOwner:AddPixie(arcMarker.tPixie)
							end
						end
					end
				end
			end
		end
		------------------------------------------ End Markers ------------------------------------------
		
		------------------------------------------ Draw Closest Windows ------------------------------------------
		local bIsInCombat = playerUnit:IsInCombat()
		for i, tMarkerFilter in pairs(self.tMarkerFilters) do
			local arcMarker = tMarkerFilter.wndClosestObj:GetData()
			local wndClosestObj = tMarkerFilter.wndClosestObj
			if arcMarker ~= nil and bIsInCombat == false then
				local nLabelWidth = 56
				local nLabelHeight = 25
				local tLocation = arcMarker.tPixie.loc.nOffsets
				local nMarkerWidth = arcMarker.nWidth
				local nMarkerHeight = arcMarker.nHeight
				local nOL = tLocation[1]+(nMarkerWidth/2)-(nLabelWidth/2)
				local nOT = 30
				local nOR = nOL+nLabelWidth
				local nOB = nOT+nLabelHeight
				local bShowPreview = false
				if arcMarker.tMarkerSpriteDisposition ~= nil then
					bShowPreview = arcMarker.tMarkerSpriteDisposition.tMarkerFilter.bShowPreview
				end
				if arcMarker.tMarkerSpriteFilter ~= nil then
					bShowPreview = arcMarker.tMarkerSpriteFilter.tMarkerFilter.bShowPreview
				end
				if arcMarker.tMarkerSpriteChallenge ~= nil then
					bShowPreview = arcMarker.tMarkerSpriteChallenge.tMarkerFilter.bShowPreview
				end
				
				if self.tToggleOptions.tUnitPreview.isToggled and bShowPreview == true then
					if arcMarker.eType == self.tMarkerTypes.Unit then
						wndClosestObj:FindChild("WindowTarget"):SetCostume(arcMarker.uOwner)
						wndClosestObj:FindChild("WindowTarget"):Show(true)
						nOB = nOT+70
					end
				else
					wndClosestObj:FindChild("WindowTarget"):Show(false)
				end
				wndClosestObj:SetAnchorOffsets(nOL, nOT, nOR, nOB)
				wndClosestObj:FindChild("WindowText"):SetText(tostring(m_floor(arcMarker.nDistance)).."m")
				wndClosestObj:Show(true)
			else
				wndClosestObj:Show(false)
			end
		end
		------------------------------------------ Draw Closest Windows ------------------------------------------

		------------------------------------------ Auto Hide Addon ------------------------------------------
		local bAutoHide = self.tToggleOptions.tAutoHide.isToggled
		if bAutoHide == true and bIsInCombat == true then
			if self.bMainWindowShown == true then
				self.wndMain:Show(false)
				self.bMainWindowShown = false
			end
		else
			if self.bMainWindowShown == false then
				self.wndMain:Show(true)
				self.bMainWindowShown = true
			end
		end
		------------------------------------------ Auto Hide Addon ------------------------------------------
	end
end


-----------------------------------------------------------------------------------------------
-- Arcompass Group EventHandler
-----------------------------------------------------------------------------------------------
function Arcompass:OnGroupUpdatePosition(tPositions)
	if not tPositions then return end
	local playerUnit = self.playerUnit
	if playerUnit ~= nil and playerUnit:IsValid() then
		local CurrentZoneMap = GameLib.GetCurrentZoneMap()
		if CurrentZoneMap ~= nil then
			local playerContinentId = GameLib.GetCurrentZoneMap().continentId
			
			for idx, tRecord in pairs(tPositions) do
				if tRecord.nIndex ~= 1 then
					local tGroupMember = GroupLib.GetGroupMember(tRecord.nIndex)
					local idMarker = "group"..tostring(tGroupMember.nMemberIdx)
					--Print(tGroupMember.strCharacterName)
					if tGroupMember.strCharacterName ~= self.playerUnit:GetTitleOrName() then
						local arcMarker = self.arcMarkersCollections[self.tMarkerFilters.tGroup.name][idMarker]
						if arcMarker == nil then
							self:OnGroupChanged()
						elseif tRecord.tWorldLoc ~= nil and playerContinentId == tRecord.tZoneMap.continentId then
							for k, tMarker in pairs(arcMarker.tMarkers) do
								tMarker.tLocation = tRecord.tWorldLoc
							end
						else
							for k, tMarker in pairs(arcMarker.tMarkers) do
								tMarker.tLocation = nil
							end
						end
					end
				end
			end
		end
	end
end

function Arcompass:OnGroupChanged()
	local playerUnit = self.playerUnit
	if playerUnit ~= nil and playerUnit:IsValid() then
		-- Just clear current path missions
		if self.arcMarkersCollections[self.tMarkerFilters.tGroup.name] ~= nil then
			for idx, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tGroup.name]) do
				self:DestroyMarker(arcMarker)
			end
			self.arcMarkersCollections[self.tMarkerFilters.tGroup.name] = {}
		end

		local nGroupMemberCount = GroupLib.GetMemberCount()
		if nGroupMemberCount > 0 then
			for idx = 2, nGroupMemberCount do
				local tGroupMember = GroupLib.GetGroupMember(idx)
				local idMarker = "group"..tostring(tGroupMember.nMemberIdx)
				if tGroupMember ~= nil then
					local uGroupMember = GroupLib.GetUnitForGroupMember(idx)
					if tGroupMember.strCharacterName ~= self.playerUnit:GetTitleOrName() then
						local arcMarker = {
							uOwner 			= uGroupMember,
							idMarker		= idMarker,
							strName			= tGroupMember.strCharacterName,
							nRelativePos	= nil,
							nDistance		= 99999,
							tClosestMarker	= nil,
							bHighlighted	= false,
							tMarkerSpriteChallenge 		= nil,
							tMarkerSpriteFilter 		= self.tMarkerSprites.GroupMember,
							tMarkerSpriteDisposition 	= nil,
							eType			= self.tMarkerTypes.Group,
							nGroupNumber	= tGroupMember.nMemberIdx,
							strDescription	= nil,
							wndOwner		= self.wndUnit,
							nHeight			= 32,
							nWidth 			= 32,
							tPixie 			= {
								loc = { 
									fPoints = { 0, 0, 0, 0 },
									nOffsets = { 0, 0, 32, 32 }
								},
								strSprite = "",
								cr = { a=1.00, r=1.00, g=1.00, b=1.00 },
								crText = { a=1, r=1.00, g=1.00, b=1.00 },
								fRotation = 0, 
								strFont = "CRB_InterfaceSmall",
								strText = "",
								flagsText = {
									DT_BOTTOM = true,
									DT_CENTER = true
								}
							},
							nPixieId 		= nil,
							tMarkers		= {{
								tLocation = nil
							}}
						}
						self.arcMarkers[idMarker] = arcMarker
						self.arcMarkersCollections[self.tMarkerFilters.tGroup.name][arcMarker.idMarker] = arcMarker
						self:ClearMarker(arcMarker)
					end
				end
			end
		end
	end
end


-----------------------------------------------------------------------------------------------
-- Arcompass Quest EventHandler
-----------------------------------------------------------------------------------------------
function Arcompass:OnQuestChanged()
	-- Just clear current path missions
	if self.arcMarkersCollections[self.tMarkerFilters.tQuestTracked.name] ~= nil then
		for idx, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tQuestTracked.name]) do
			self:DestroyMarker(arcMarker)
		end
		self.arcMarkersCollections[self.tMarkerFilters.tQuestTracked.name] = {}
	end

	-- Rebuild
	local tEpisodeList = QuestLib.GetTrackedEpisodes()
	local tCurrentZoneMap = GameLib.GetCurrentZoneMap()
	if tEpisodeList == nil or self.tWorldZone == nil or tCurrentZoneMap == nil then return end
	if tCurrentZoneMap.strFolder == nil or tCurrentZoneMap.strName == nil then return end
	for idx, uEpisode in ipairs(tEpisodeList) do
	local strZoneName = uEpisode:GetZoneName()
		local bZoneInMapName = string.find(tCurrentZoneMap.strName, strZoneName)
		local bZoneInMapFolder = string.find(tCurrentZoneMap.strFolder, strZoneName)
		if bZoneInMapFolder ~= nil or bZoneInMapName ~= nil then
			for idx2, uQuest in ipairs(uEpisode:GetTrackedQuests()) do
				-- Build marker set
				local idOwner = uQuest:GetId()
				local idMarker = "quest"..tostring(idk)..tostring(idOwner)
				local arcMarker = {
					uOwner 			= uQuest,
					idMarker 		= idMarker,
					strName 		= uQuest:GetTitle(),
					nRelativePos	= nil,
					nDistance		= 99999,
					tClosestMarker	= nil,
					bHighlighted	= false,
					tMarkerSpriteFilter 	= self.tMarkerSprites.QuestArrow,
					tMarkerSpriteChallenge 		= nil,
					tMarkerSpriteDisposition 	= nil,
					eType 			= self.tMarkerTypes.Quest,
					strDescription	= uQuest:GetSummary(),
					wndOwner		= self.wndQuest,
					nHeight			= 48,
					nWidth 			= 48,
					tPixie 			= {
						loc = { 
							fPoints = { 0, 0, 0, 0 },
							nOffsets = { 0, 0, 32, 32 }
						},
						strSprite = self.tMarkerSprites.QuestArrow.strSprite,
						cr = { a=1.00, r=1.00, g=1.00, b=1.00 },
						crText = { a=1, r=0.00, g=1.00, b=0.50 },
						fRotation = 0, 
						strFont = "CRB_InterfaceSmall",
						strText = "",
						flagsText = {
							DT_VCENTER = true,
							DT_CENTER = true
						}
					},
					nPixieId 		= nil,
					tMarkers		= {}
				}
				local tMRs = uQuest:GetMapRegions()
				for idk, uRegion in pairs(tMRs) do
					arcMarker.tMarkers[idk] = {
						tLocation = uRegion.tIndicator
					}
				end

				-- Load the markers into a mission object and store for later destruction
				self.arcMarkers[arcMarker.idMarker] = arcMarker
				self.arcMarkersCollections[arcMarker.tMarkerSpriteFilter.tMarkerFilter.name][arcMarker.idMarker] = arcMarker
				self:ClearMarker(arcMarker)
			end
		end
	end
end

function Arcompass:OnQuestStateChanged()
	self:OnQuestChanged()
	
	-- Refresh markers that may have had their rewardinfo changed
	for i, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tFriendly.name]) do
		self:RefreshMarker(arcMarker)
	end
	for i, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tNeutral.name]) do
		self:RefreshMarker(arcMarker)
	end
	for i, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tHostile.name]) do
		self:RefreshMarker(arcMarker)
	end
end

function Arcompass:OnQuestObjectiveChanged()
	self:OnQuestChanged()

	-- Refresh markers that may have had their rewardinfo changed
	for i, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tQuestTargets.name]) do
		self:RefreshMarker(arcMarker)
	end
	for i, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tNeutral.name]) do
		self:RefreshMarker(arcMarker)
	end
	for i, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tHostile.name]) do
		self:RefreshMarker(arcMarker)
	end
end

-----------------------------------------------------------------------------------------------
-- Arcompass Mission Events/Functions
-----------------------------------------------------------------------------------------------
function Arcompass:OnPlayerPathMissionUpdate()
	-- Just clear current path missions
	if self.arcMarkersCollections[self.tMarkerFilters.tPathMissions.name] ~= nil then
		for idx, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tPathMissions.name]) do
			self:DestroyMarker(arcMarker)
		end
		self.arcMarkersCollections[self.tMarkerFilters.tPathMissions.name] = {}
	end

	-- Rebuild
	local tMarkerSpriteFilter = self.playerPathMarker
	local tPathEpisodes = PlayerPathLib.GetPathEpisodeForZone()
	if tPathEpisodes == nil or self.tWorldZone == nil then return end
	for kMission, uMission in ipairs(tPathEpisodes:GetMissions()) do
		-- We don't care about missions complete or blank
		local isComplete = uMission:IsComplete()
		local name = uMission:GetName()
		if isComplete == false or name ~= "" then
			-- Build marker set
			local idOwner = uMission:GetId()
			local idMarker = "path"..tostring(idk)..tostring(idOwner)
			local arcMarker = {
				uOwner 			= uMission,
				idMarker 		= idMarker,
				strName 		= uMission:GetName(),
				nRelativePos	= nil,
				nDistance		= nil,
				tClosestMarker	= nil,
				bHighlighted	= false,
				tMarkerSpriteChallenge 		= nil,
				tMarkerSpriteFilter 		= tMarkerSpriteFilter,
				tMarkerSpriteDisposition 	= nil,
				eType 			= self.tMarkerTypes.Path,
				strDescription	= uMission:GetSummary(),
				wndOwner		= self.wndQuest,
				nHeight			= 48,
				nWidth 			= 48,
				tPixie 			= {
					loc = { 
						fPoints = { 0, 0, 0, 0 },
						nOffsets = { 0, 0, 32, 32 }
					},
					strSprite = tMarkerSpriteFilter.strSprite,
					cr = { a=1.00, r=1.00, g=1.00, b=1.00 },
					crText = { a=1, r=1.00, g=1.00, b=1.00 },
					fRotation = 0, 
					strFont = "Default",
					strText = "",
					flagsText = {
						DT_BOTTOM = true,
						DT_CENTER = true
					}
				},
				nPixieId 		= nil,
				tMarkers		= {}
			}
			local tMRs = uMission:GetMapRegions()
			for idk, uRegion in pairs(tMRs) do
				arcMarker.tMarkers[idk] = {
					tLocation = uRegion.tIndicator
				}
			end

			-- Load the markers into a mission object and store for later destruction
			self.arcMarkers[arcMarker.idMarker] = arcMarker
			self.arcMarkersCollections[arcMarker.tMarkerSpriteFilter.tMarkerFilter.name][arcMarker.idMarker] = arcMarker
			self:ClearMarker(arcMarker)
		end
	end
end


-----------------------------------------------------------------------------------------------
-- Arcompass Challenges Events/Functions
-----------------------------------------------------------------------------------------------
function Arcompass:OnChallengeUpdated()
	--Print("OnChallengeUpdated!")
	-- Refresh markers that may have had their rewardinfo changed
	for i, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tChallengeTargets.name]) do
		self:RefreshMarker(arcMarker)
	end
	for i, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tNeutral.name]) do
		self:RefreshMarker(arcMarker)
	end
	for i, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tHostile.name]) do
		self:RefreshMarker(arcMarker)
	end
end

function Arcompass:OnChallengeInitEnd()
	self:OnUpdateChallenges()
	self:OnChallengeUpdated()
end

function Arcompass:OnUpdateChallenges()
	-- Just clear current path missions
	if self.arcMarkersCollections[self.tMarkerFilters.tChallenges.name] ~= nil then
		for idx, arcMarker in pairs(self.arcMarkersCollections[self.tMarkerFilters.tChallenges.name]) do
			self:DestroyMarker(arcMarker)
		end
		self.arcMarkersCollections[self.tMarkerFilters.tChallenges.name] = {}
	end

	-- Rebuild
	local tCurrentZoneMap = GameLib.GetCurrentZoneMap()
	if tCurrentZoneMap == nil then return end
	if tCurrentZoneMap.strFolder == nil or tCurrentZoneMap.strName == nil then return end
	local tChallenges = ChallengesLib.GetActiveChallengeList()
	for i, uChallenge in pairs(tChallenges) do
		local tZoneInfo = uChallenge:GetZoneInfo()
		local bZoneInMapName = string.find(tCurrentZoneMap.strName, tZoneInfo.strZoneName)
		local bZoneInMapFolder = string.find(tCurrentZoneMap.strFolder, tZoneInfo.strZoneName)
		if bZoneInMapFolder == 1 or bZoneInMapName == 1 then
			if uChallenge:IsInCooldown() == false and uChallenge:IsFullyComplete() == false then
				local idMarker = "challenge"..tostring(uChallenge:GetId())
				local arcMarker = {
					uOwner 			= uChallenge,
					idMarker		= idMarker,
					strName			= uChallenge:GetName(),
					nRelativePos	= nil,
					nDistance		= 99999,
					tClosestMarker	= nil,
					bHighlighted	= false,
					tMarkerSpriteChallenge 		= nil,
					tMarkerSpriteFilter 		= self.tMarkerSprites.Challenges,
					tMarkerSpriteDisposition 	= nil,
					eType			= self.tMarkerTypes.Challenge,
					strDescription	= uChallenge:GetDescription(),
					wndOwner		= self.wndQuest,
					nHeight			= 32,
					nWidth 			= 32,
					tPixie 			= {
						loc = { 
							fPoints = { 0, 0, 0, 0 },
							nOffsets = { 0, 0, 32, 32 }
						},
						strSprite = "",
						cr = { a=1.00, r=1.00, g=1.00, b=1.00 },
						crText = { a=1, r=1.00, g=1.00, b=1.00 },
						fRotation = 0, 
						strFont = "CRB_InterfaceSmall",
						strText = "",
						flagsText = {
							DT_BOTTOM = true,
							DT_CENTER = true
						}
					},
					nPixieId 		= nil,
					tMarkers		= {{
						tLocation = uChallenge:GetMapLocation()
					}}
				}
				
				self.arcMarkers[idMarker] = arcMarker
				self.arcMarkersCollections[arcMarker.tMarkerSpriteFilter.tMarkerFilter.name][arcMarker.idMarker] = arcMarker
				self:ClearMarker(arcMarker)
			end
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Arcompass Unit Events/Functions
-----------------------------------------------------------------------------------------------
function Arcompass:OnUnitCreated(unitNew)
	if unitNew == nil or not unitNew:IsValid() then return end

	local idOwner = unitNew:GetId()
	if idOwner == nil then return end
	
	-- Backlog the creation of this unit until player has loaded
	if (self.playerUnit == nil or self.playerGuild == nil or not GameLib.IsCharacterLoaded()) then
		if self.arcUnitBacklog[idOwner] == nil then
			self.arcUnitBacklog[idOwner] = unitNew
			return
		end
	end
	
	-- Check this just isn't the player unit...
	if unitNew == GameLib.GetPlayerUnit() then return end
	local idMarker = "unit"..tostring(idOwner)
	
	local arcMarker = self.arcMarkers[idMarker]
	if arcMarker ~= nil then
		if arcMarker.uOwner ~= nil then
			self:RefreshMarker(arcMarker)
		end
	else
		local idMarker = "unit"..tostring(unitNew:GetId())
		local arcMarker = {
			uOwner 			= unitNew,
			idMarker		= idMarker,
			strName			= unitNew:GetTitleOrName(),
			nRelativePos	= nil,
			nDistance		= 99999,
			tClosestMarker	= nil,
			bHighlighted	= false,
			tMarkerSpriteChallenge 		= nil,
			tMarkerSpriteFilter 		= nil,
			tMarkerSpriteDisposition 	= nil,
			eType			= self.tMarkerTypes.Unit,
			strDescription	= nil,
			wndOwner		= self.wndUnit,
			nHeight			= 32,
			nWidth 			= 32,
			tPixie 			= {
				loc = { 
					fPoints = { 0, 0, 0, 0 },
					nOffsets = { 0, 0, 32, 32 }
				},
				strSprite = "",
				cr = { a=1.00, r=1.00, g=1.00, b=1.00 },
				crText = { a=1, r=1.00, g=1.00, b=1.00 },
				fRotation = 0, 
				strFont = "CRB_InterfaceSmall",
				strText = "",
				flagsText = {
					DT_BOTTOM = true,
					DT_CENTER = true
				}
			},
			nPixieId 		= nil,
			tMarkers		= {{
				tLocation = nil
			}}
		}
		
		self:RefreshMarker(arcMarker)
		if arcMarker.tMarkerSpriteDisposition == nil then 
			return 
		end
		self.arcMarkers[idMarker] = arcMarker
	end
end


function Arcompass:OnUnitDestroyed(unitDestroyed)
	local idOwner = unitDestroyed:GetId()
	if idOwner ~= nil then
		--Print(unitDestroyed:GetName())
		local idMarker = "unit"..tostring(idOwner)
		if self.arcMarkers[idMarker] ~= nil then
			self:DestroyMarker(self.arcMarkers[idMarker])
		end
	end
end


-----------------------------------------------------------------------------------------------
-- Arcompass Common Marker Functions
-----------------------------------------------------------------------------------------------
function Arcompass:RefreshMarker(arcMarker)
	if arcMarker.uOwner:IsDead() == true then 
		self:DestroyMarker(arcMarker)
		return
	end
	
	local tMarkerSpriteChallenge = self:ParseChallenge(arcMarker.uOwner)
	local tMarkerSpriteFilter = self:ParseReward(arcMarker.uOwner)
	local tMarkerSpriteDisposition = self:ParseDisposition(arcMarker.uOwner)
	
	if tMarkerSpriteDisposition == nil then 
		self:DestroyMarker(arcMarker)
		return
	end
	
	-- Update Challenge
	if tMarkerSpriteChallenge ~= nil then
		arcMarker.tMarkerSpriteChallenge = tMarkerSpriteChallenge
		self.arcMarkersCollections[arcMarker.tMarkerSpriteChallenge.tMarkerFilter.name][arcMarker.idMarker] = arcMarker
	else
		if arcMarker.tMarkerSpriteChallenge ~= nil then
			self.arcMarkersCollections[arcMarker.tMarkerSpriteChallenge.tMarkerFilter.name][arcMarker.idMarker] = nil
			arcMarker.tMarkerSpriteChallenge = nil
		end
	end
	-- Update filter
	if tMarkerSpriteFilter ~= nil then
		if arcMarker.tMarkerSpriteFilter ~= nil then
			if arcMarker.tMarkerSpriteFilter.tMarkerFilter.name ~= tMarkerSpriteFilter.tMarkerFilter.name then
				self.arcMarkersCollections[arcMarker.tMarkerSpriteFilter.tMarkerFilter.name][arcMarker.idMarker] = nil
			end
		end
		arcMarker.tMarkerSpriteFilter = tMarkerSpriteFilter
		self.arcMarkersCollections[arcMarker.tMarkerSpriteFilter.tMarkerFilter.name][arcMarker.idMarker] = arcMarker
	else
		if arcMarker.tMarkerSpriteFilter ~= nil then
			self.arcMarkersCollections[arcMarker.tMarkerSpriteFilter.tMarkerFilter.name][arcMarker.idMarker] = nil
			arcMarker.tMarkerSpriteFilter = nil
		end
	end
	-- Update disposition
	if tMarkerSpriteDisposition ~= nil then
		if arcMarker.tMarkerSpriteDisposition ~= nil then
			if tMarkerSpriteDisposition.tMarkerFilter.name ~= arcMarker.tMarkerSpriteDisposition.tMarkerFilter.name then
				self.arcMarkersCollections[arcMarker.tMarkerSpriteDisposition.tMarkerFilter.name][arcMarker.idMarker] = nil
			end
		end
		arcMarker.tMarkerSpriteDisposition = tMarkerSpriteDisposition
		self.arcMarkersCollections[arcMarker.tMarkerSpriteDisposition.tMarkerFilter.name][arcMarker.idMarker] = arcMarker
	else
		if arcMarker.tMarkerSpriteDisposition ~= nil then
			self.arcMarkersCollections[arcMarker.tMarkerSpriteDisposition.tMarkerFilter.name][arcMarker.idMarker] = nil
			arcMarker.tMarkerSpriteDisposition = nil
		end
	end
	
	self:ClearMarker(arcMarker)
end

function Arcompass:DestroyMarker(arcMarker)
	self:ClearMarker(arcMarker)

	if arcMarker.tMarkerSpriteChallenge ~= nil then
		self.arcMarkersCollections[arcMarker.tMarkerSpriteChallenge.tMarkerFilter.name][arcMarker.idMarker] = nil
	end
	if arcMarker.tMarkerSpriteFilter ~= nil then
		self.arcMarkersCollections[arcMarker.tMarkerSpriteFilter.tMarkerFilter.name][arcMarker.idMarker] = nil
	end
	if arcMarker.tMarkerSpriteDisposition ~= nil then
		self.arcMarkersCollections[arcMarker.tMarkerSpriteDisposition.tMarkerFilter.name][arcMarker.idMarker] = nil
	end
	self.arcMarkers[arcMarker.idMarker] = nil
	arcMarker = nil
end


function Arcompass:ClearMarker(arcMarker)
	if arcMarker.nPixieId ~= nil then
		local idMarker = arcMarker.idMarker
		arcMarker.wndOwner:DestroyPixie(arcMarker.nPixieId)
		arcMarker.nPixieId = nil
	end
	self.arcMarkersDraw[arcMarker.idMarker] = nil
end

function Arcompass:ParseChallenge(unit)
	local tRewardInfo = unit:GetRewardInfo()
	if tRewardInfo and type(tRewardInfo) == "table" then
		local count = #tRewardInfo
		for idx = 1, count do
			local idChallenge = tRewardInfo[idx].idChallenge
			if idChallenge ~= nil then
				local uChallenge = ChallengesLib.GetActiveChallengeList()[idChallenge]
				if uChallenge ~= nil then
					if uChallenge:IsActivated() then
						return self.tMarkerSprites.ChallengeTargets
					end
				end
			end
		end
	end
	
	return nil
end


function Arcompass:ParseDisposition(unit)
	local unitDisposition = unit:GetDispositionTo(self.playerUnit)
	if unitDisposition == 0 then
		if not unit:IsDead() then
			return self.tMarkerSprites.Hostile
		end
	elseif unitDisposition == 1 then
		if not unit:IsDead() then
			return self.tMarkerSprites.Neutral
		end
	elseif unitDisposition == 2 then
		return self.tMarkerSprites.Friendly
	end
	
	return nil
end


function Arcompass:ParseReward(unit)
	-- Dropped Weapon
	if unit:GetType() == "Pickup" then
		local playerName = self.playerUnit:GetName()
		if string.sub(unit:GetName(), 1, string.len(playerName)) == playerName then
			return self.tMarkerSprites.WeaponDrop
		end
	end
	
	-- Quest NPCs
	local tActivation = unit:GetActivationState()
	if tActivation.QuestTarget ~= nil then
		if tActivation.Busy == nil and tActivation.Interact ~= nil then
			return self.tMarkerSprites.QuestTarget
		end
		if tActivation.TalkTo ~= nil then
			return self.tMarkerSprites.TalkTo
		end
	elseif tActivation.QuestReward ~= nil then
		return self.tMarkerSprites.QuestReward
	elseif tActivation.QuestNewTradeskill ~= nil then
		return self.tMarkerSprites.QuestNewTradeskill
	elseif tActivation.QuestNew ~= nil or tActivation.QuestNewMain ~= nil or tActivation.QuestNewRepeatable ~= nil then
		return self.tMarkerSprites.QuestNew
	elseif tActivation.QuestReceiving ~= nil then
		return nil --tMarkerSprites.QuestReceiving
	elseif tActivation.TalkTo ~= nil then
		return self.tMarkerSprites.TalkTo
	end
	
	-- Quest Target/Creature
	local unitDisposition = unit:GetDispositionTo(self.playerUnit)
	if unit:IsDead() ~= true then
		tRewardInfo = unit:GetRewardInfo()
		if tRewardInfo and type(tRewardInfo) == "table" then
			local count = #tRewardInfo
			for idx = 1, count do
				--[[local questId = tRewardInfo[idx].idQuest
				if questId ~= nil then
					if self.arcMarkersCollections[self.tMarkerFilters.tQuestTracked.name]["quest"..tostring(questId)] ~= nil then
						local nCompleted = tRewardInfo[idx].nCompleted
						local nNeeded = tRewardInfo[idx].nNeeded
						if nNeeded < nCompleted then
							if unitDisposition == 0 or unitDisposition == 1 then
								if unit:GetHealth() ~= nil and not unit:IsDead() then
									return self.tMarkerSprites.QuestCreature
								else
									return self.tMarkerSprites.QuestTarget
								end
							else
								if tActivation.QuestTarget ~= nil then
									if tActivation.QuestTarget.bCanInteract == true then
										return self.tMarkerSprites.QuestTarget
									end
								else
									if tActivation.Busy == nil and tActivation.Interact ~= nil then
										return self.tMarkerSprites.QuestTarget
									end
								end
							end
						end
					else
						-- Inactive?
					end
				end]]--
				
				local type = tRewardInfo[idx].strType
				if type == "Quest" then
					if unit:GetHealth() ~= nil and unit:GetHealth() ~= 0 and
						(unitDisposition == 0 or unitDisposition == 1) then
						return self.tMarkerSprites.QuestCreature
					else
						local next = next
						if next(tActivation) ~= nil then
							if tActivation.Busy == nil then --and tActivation.Interact ~= nil then
								if tActivation.QuestTarget ~= nil then
									if tActivation.Interact ~= nil then
										if tActivation.Interact.bCanInteract == true then
											return self.tMarkerSprites.QuestTarget
										end
									end	
								else
									return self.tMarkerSprites.QuestTarget
								end
							end
						else
							return self.tMarkerSprites.QuestTarget
						end
					end
				end
			end
		end
	end
	
	-- Scientist
	if tActivation.ScientistScannable ~= nil then
		return self.tMarkerSprites.Scientist_Scannable
	end
	if tActivation.ScientistRawScannable then
		return self.tMarkerSprites.Scientist_Raw
	end
	
	-- General NPCs/PCs
	if unitDisposition == 0 then -- Hostile
		if unit:IsRival() then return self.tMarkerSprites.Rival end -- Rival
	elseif unitDisposition == 1 or unitDisposition == 2 then -- Neutral/Friendly
		local tHarvestType = unit:GetHarvestRequiredTradeskillName()
		if tHarvestType == L["TradeskillMining"] then
			return self.tMarkerSprites.MiningNodes
		elseif tHarvestType == L["TradeskillSurvivalist"] then
			return self.tMarkerSprites.SurvivalistNodes
		elseif tHarvestType == L["TradeskillRelicHunter"] then
			return self.tMarkerSprites.RelicNodes
		elseif tHarvestType == L["TradeskillFarming"] then
			return self.tMarkerSprites.FarmingNodes
		end
		
		if tActivation.CommodityMarketplace ~= nil then
			return self.tMarkerSprites.CommodityMarketplace	
		elseif tActivation.ItemAuctionhouse ~= nil then
			return self.tMarkerSprites.ItemAuctionhouse
		elseif tActivation.Dye ~= nil then
			return self.tMarkerSprites.Dye
		elseif tActivation.Vendor ~= nil then
			--local strAffiliation = unit:GetAffiliationName()
			--if strAffiliation == "<Armor Vendor>" then
			--	return self.tMarkerSprites.VendorArmor
			--elseif strAffiliation == "<Tradeskill Goods Vendor>" or strAffiliation == "<Cooking Trainer>" then
			--	return self.tMarkerSprites.VendorTradeskill
			--end
			return self.tMarkerSprites.VendorGeneral
		elseif tActivation.CraftingStation ~= nil then
			return self.tMarkerSprites.CraftingStation
		elseif tActivation.Tradeskill ~= nil then
			return self.tMarkerSprites.Tradeskill
		elseif tActivation.TradeskillTrainer ~= nil then
			return self.tMarkerSprites.Trainer
		elseif tActivation.BindPoint ~= nil or 
			   tActivation.BindPointCurrent ~= nil then
			return self.tMarkerSprites.BindPoint
		elseif tActivation.FlightPath ~= nil or 
			   tActivation.FlightPathSettler ~= nil or 
			   tActivation.FlightPathNew then
			return self.tMarkerSprites.FlightPath
		elseif tActivation.InstancePortal ~= nil then
			return self.tMarkerSprites.InstancePortal
		elseif tActivation.Datacube ~= nil then
			return self.tMarkerSprites.Datacube
		elseif tActivation.Mail ~= nil then
			return self.tMarkerSprites.Mail
		elseif tActivation.Bank ~= nil then
			return self.tMarkerSprites.Bank
		elseif tActivation.GuildBank ~= nil then
			return self.tMarkerSprites.GuildBank
		end
		
		-- Settler/Explorer
		if tActivation.Interact ~= nil and tActivation.Collect ~= nil then
			if tActivation.Collect.bUsePlayerPath == true then
				return self.tMarkerSprites.SettlerCollect
			else
				return self.tMarkerSprites.ChallengeTargets -- Not 100% sure
			end
		end
		local next = next
		if next(tActivation) ~= nil then
			if tActivation.Busy == nil then
				if tActivation.ExplorerActivate ~= nil then
					return self.tMarkerSprites.Explorer
				end
				if tActivation.SettlerActivate ~= nil then 
					return self.tMarkerSprites.SettlerCollect 
				end
				local affiliation = unit:GetAffiliationName()
				if affiliation == L["ExplorersUnion"] then 
					if self.playerPathMarker == self.tMarkerSprites.Path_Explorer then
						return self.tMarkerSprites.Explorer 
					end
				end
				if tActivation.Spell ~= nil then
					local name = unit:GetName()
					if name == "Discovery" then
						--Do something
					elseif name == "Unusual Rock" then
						--Blah
					end
				end
			end
		end
		-- Soldier?
		
		--Print(unit:GetTitleOrName().." "..tostring(unit:GetGuildName()))
		if (unit:IsFriend() or (unit:GetGuildName() == self.playerGuild and unit:GetGuildName() ~= nil)) and
			not unit:IsInYourGroup() then -- Friend/Guild Member
			return self.tMarkerSprites.Friend
		end
	end
end


---------------------------------------------------------------------------------------------------
-- ArcompassOptions Events
---------------------------------------------------------------------------------------------------
function Arcompass:OnFilterOptionUncheck( wndHandler, wndControl, eMouseButton )
	local tMarkerFilter = wndControl:GetData()
	if tMarkerFilter == nil then return end
	
	tMarkerFilter.isToggled = false
	
	local bUncheck = false
	
	for i, arcMarker in pairs(self.arcMarkersCollections[tMarkerFilter.name]) do
		if arcMarker.tMarkerSpriteFilter ~= nil then
			if arcMarker.tMarkerSpriteFilter.tMarkerFilter == tMarkerFilter then
				bUncheck = true
			end
		end
		if arcMarker.tMarkerSpriteChallenge ~= nil then
			if arcMarker.tMarkerSpriteChallenge.tMarkerFilter == tMarkerFilter then
				bUncheck = true
			end
		end
		if arcMarker.tMarkerSpriteDisposition ~= nil then
			if arcMarker.tMarkerSpriteDisposition.tMarkerFilter == tMarkerFilter then
				bUncheck = true
			end
		end
		
		if bUncheck then
			self:ClearMarker(arcMarker)
		end
	end
end

function Arcompass:OnFilterOptionCheck( wndHandler, wndControl, eMouseButton )
	local tMarkerFilter = wndControl:GetData()
	if tMarkerFilter == nil then return end
	
	tMarkerFilter.isToggled = true
	--for i, arcMarker in pairs(self.arcMarkersCollections[tMarkerFilter.name]) do
	--	local idMarker = arcMarker.idMarker
	--	self.arcMarkersDraw[idMarker] = arcMarker
	--end		
end

function Arcompass:OnOptionsCheck( wndHandler, wndControl, eMouseButton )
	local tToggleOption = wndControl:GetData()
	if tToggleOption == nil then return end

	tToggleOption.isToggled = true
	self:UpdateOptions(tToggleOption)
end

function Arcompass:OnOptionsUncheck( wndHandler, wndControl, eMouseButton )
	local tToggleOption = wndControl:GetData()
	if tToggleOption == nil then return end

	tToggleOption.isToggled = false
	self:UpdateOptions(tToggleOption)
end

function Arcompass:UpdateOptions( tToggleOption )
	if tToggleOption.name == "LockFrame" then
		-- Lock Frame
		local btnLockFrame = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tLockFrame.name)
		local bLockFrame = self.tToggleOptions.tLockFrame.isToggled
		self.wndMain:SetStyle("Moveable", not bLockFrame)
		self.wndMain:SetStyle("Sizable", not bLockFrame)
		--self.wndMain:FindChild("ResizeLeft"):Show(not bLockFrame)
		--self.wndMain:FindChild("ResizeRight"):Show(not bLockFrame)
		btnLockFrame:SetTooltip("Lock Frame "..(bLockFrame and "[LOCKED]" or "[UNLOCKED]"))
	elseif tToggleOption.name == "ZoneText" then
		-- Zone Text
		local btnZoneText = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tZoneText.name)
		local bZoneText = self.tToggleOptions.tZoneText.isToggled
		btnZoneText:SetTooltip("Show Zone Info "..(bZoneText and "[ON]" or "[OFF]"))
		self.wndMain:FindChild("ZoneText"):Show(bZoneText)
		self.wndMain:FindChild("PlayerCoord"):Show(bZoneText)
	elseif tToggleOption.name == "Background" then	
		-- Background
		local btnBackground = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tBackground.name)
		local bBackground = self.tToggleOptions.tBackground.isToggled
		btnBackground:SetTooltip("Show Compass BG "..(bBackground and "[ON]" or "[OFF]"))
		self.wndMain:FindChild("ArcompassBackground"):Show(bBackground)
	elseif tToggleOption.name == "PlayerMarker" then	
		-- Player Marker
		local btnPlayerMarker = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tPlayerMarker.name)
		local bPlayerMarker = self.tToggleOptions.tPlayerMarker.isToggled
		btnPlayerMarker:SetTooltip("Show Player Marker "..(bPlayerMarker and "[ON]" or "[OFF]"))
		local wndPlayerMarker = self.wndMain:FindChild("PlayerMarker")
		wndPlayerMarker:Show(bPlayerMarker)
		wndPlayerMarker:SetOpacity(self.tToggleOptions.tPlayerMarker.opSetting*0.01)
	elseif tToggleOption.name == "UnitPreview" then
		local btnUnitPreview = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tUnitPreview.name)
		local bUnitPreview = self.tToggleOptions.tUnitPreview.isToggled
		btnUnitPreview:SetTooltip("Show Unit Portraits "..(bUnitPreview and "[ON]" or "[OFF]"))
	elseif tToggleOption.name == "AutoHide" then
		local btnAutoHide = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tAutoHide.name)
		local bAutoHide = self.tToggleOptions.tAutoHide.isToggled
		btnAutoHide:SetTooltip("Auto Hide Feature "..(bAutoHide and "[ON]" or "[OFF]"))
	elseif tToggleOption.name == "Opacity" then
		-- Opacity Button
		local btnOpacity = self.wndArcompassOptions:FindChild("OptionsBtn"..self.tToggleOptions.tOpacity.name)
		local bOpacity = self.tToggleOptions.tOpacity.isToggled
		btnOpacity:SetTooltip("Show Opacity Sliders "..(bOpacity and "[ON]" or "[OFF"))
		for i, tMarkerFilter in pairs(self.tMarkerFilters) do
			if tMarkerFilter.name ~= self.tMarkerFilters.tWeaponDrop.name and 
				tMarkerFilter.name ~= self.tMarkerFilters.tFriendly.name then
				-- Set Filter Button
				local wndToggleBtn = self.wndArcompassOptions:FindChild("OptionsBtn"..tMarkerFilter.name)
				-- Set Filter Background
				local wndSliderBackground = wndToggleBtn:FindChild("SliderBG")
				wndSliderBackground:SetStyle("IgnoreMouse", bOpacity)
				wndSliderBackground:Show(bOpacity)
				if bOpacity then
					wndToggleBtn:SetText("")
				else
					wndToggleBtn:SetText("          "..tMarkerFilter.buttonStr)
				end
			end
		end
		-- Set Compass Bar and Player Marker sliders
		for i, tToggleOption in pairs(self.tToggleOptions) do
			if tToggleOption.opSetting ~= nil then
				-- Set Filter Background
				local wndToggleBtn = self.wndArcompassOptions:FindChild("OptionsBtn"..tToggleOption.name)
				local wndSliderBackground = wndToggleBtn:FindChild("SliderBG")
				wndSliderBackground:SetStyle("IgnoreMouse", bOpacity)
				wndSliderBackground:Show(bOpacity)
				if bOpacity then
					wndToggleBtn:SetText("")
				else
					wndToggleBtn:SetText("          "..tToggleOption.buttonStr)
				end
			end
		end
	end
end

function Arcompass:OnButtonCenter( wndHandler, wndControl, eMouseButton )
	local tAnchors = {}
	tAnchors.nL, tAnchors.nT, tAnchors.nR, tAnchors.nB = self.wndMain:GetAnchorOffsets()
	local wndWidth = self.wndMain:GetWidth()
	local nScreenCenter = Apollo.GetScreenSize() / 2
	tAnchors.nL =  nScreenCenter - (wndWidth / 2)
	tAnchors.nR = tAnchors.nL  + wndWidth
	self.wndMain:SetAnchorOffsets(tAnchors.nL, tAnchors.nT, tAnchors.nR, tAnchors.nB)
end

local function Mouse2Marker(self)
	local tAnchors = {}
	tAnchors.nL, tAnchors.nT, tAnchors.nR, tAnchors.nB = self.wndMain:GetAnchorOffsets()
	local tMouse = Apollo.GetMouse()
	local nLastRelativeMouseX = tMouse.x-tAnchors.nL+16
	local idMarker = nil
	local nClosest = 99999
	local screenWidth, screenHeight = Apollo.GetScreenSize()

	for i, arcMarker in pairs(self.arcMarkersDraw) do
		if arcMarker.nRelativePos ~= nil then
			local offset = arcMarker.nWidth*0.15
			local nOL = arcMarker.tPixie.loc.nOffsets[1]+offset
			local nOR = arcMarker.tPixie.loc.nOffsets[3]-offset
			if nOL+5 < nLastRelativeMouseX and nOR > nLastRelativeMouseX then
				local nDistance = (nOL+(arcMarker.nWidth/2))-nLastRelativeMouseX
				if nDistance < nClosest then
					idMarker = arcMarker.idMarker
				else
					arcMarker.bHighlighted = false
					self.wndTooltip:Show(false)
					self.wndTooltipAlt:Show(false)
					self.wndTooltipAltText:Show(false)
				end
			else
				arcMarker.bHighlighted = false
				self.wndTooltip:Show(false)
				self.wndTooltipAlt:Show(false)
				self.wndTooltipAltText:Show(false)
			end
		end
	end
	
	if idMarker ~= nil then
		local arcMarker = self.arcMarkers[idMarker]
		arcMarker.bHighlighted = true
		
		-- Show Tooltip
		if arcMarker.eType == self.tMarkerTypes.Quest or
			arcMarker.eType == self.tMarkerTypes.Path or
			arcMarker.eType == self.tMarkerTypes.Challenge then
			tNewPosition = {}
			tNewPosition.nOL = tMouse.x-(350/2)
			tNewPosition.nOT = tAnchors.nT+15
			tNewPosition.nOR = tNewPosition.nOL+(350)
			tNewPosition.nOB = tNewPosition.nOT+(200)
			-- Modify if outside the screen
			if tNewPosition.nOR > screenWidth then
				local diff = tNewPosition.nOR - screenWidth
				tNewPosition.nOR = tNewPosition.nOR - diff
				tNewPosition.nOL = tNewPosition.nOL - diff
			end
			if tNewPosition.nOB > screenHeight then
				tNewPosition.nOT = tMouse.y-350-25
				tNewPosition.nOB = tMouse.y-25
			end
			if tNewPosition.nOL < 0 then
				local diff = tNewPosition.nOL
				tNewPosition.nOL = tNewPosition.nOL-diff+15
				tNewPosition.nOR = tNewPosition.nOR-diff+15
			end
			local wndTooltip = self.wndTooltip
			local wndTooltipText = wndTooltip:FindChild("EpisodeSummaryProgText")
			self.wndTooltipPixie.strText = arcMarker.strName
			wndTooltipText:SetAML("<P>"..arcMarker.strDescription.."</P>")
			wndTooltipText:SetHeightToContentHeight()
			wndTooltip:UpdatePixie(self.wndTooltipPixieId, self.wndTooltipPixie)
			wndTooltip:SetAnchorOffsets(tNewPosition.nOL, tNewPosition.nOT, tNewPosition.nOR, tNewPosition.nOB)
			wndTooltip:Show(true)
			wndTooltip:ToFront()
		end

		-- Show Tooltip alt
		local wndTooltipAlt = self.wndTooltipAlt
		local nLabelWidth = 56
		local nLabelHeight = 25
		local nMarkerWidth = arcMarker.nWidth
		local nMarkerHeight = arcMarker.nHeight
		local nOL = tMouse.x-(nLabelWidth/2)
		local nOT = tAnchors.nT+20
		local nOR = nOL+nLabelWidth
		local nOB = nOT+nLabelHeight
		if self.tToggleOptions.tUnitPreview.isToggled then
			local bShowPreview = false
			if arcMarker.tMarkerSpriteDisposition ~= nil then
				bShowPreview = arcMarker.tMarkerSpriteDisposition.tMarkerFilter.bShowPreview
			end
			if arcMarker.tMarkerSpriteFilter ~= nil then
				bShowPreview = arcMarker.tMarkerSpriteFilter.tMarkerFilter.bShowPreview
			end
			if arcMarker.tMarkerSpriteChallenge ~= nil then
				bShowPreview = arcMarker.tMarkerSpriteChallenge.tMarkerFilter.bShowPreview
			end
			if arcMarker.eType == self.tMarkerTypes.Unit and bShowPreview == true then	
				wndTooltipAlt:FindChild("WindowTarget"):SetCostume(arcMarker.uOwner)
				wndTooltipAlt:FindChild("WindowTarget"):Show(true)
				nOB = nOT+70
			else
				wndTooltipAlt:FindChild("WindowTarget"):Show(false)
			end
		else
			wndTooltipAlt:FindChild("WindowTarget"):Show(false)
		end
		wndTooltipAlt:SetAnchorOffsets(nOL, nOT, nOR, nOB)
		wndTooltipAlt:FindChild("WindowText"):SetText(tostring(math.floor(arcMarker.nDistance)).."m")
		wndTooltipAlt:Show(true)
		wndTooltipAlt:ToFront()

		-- Show Tooltip Text
		local wndTooltipAltText = self.wndTooltipAltText
		local strText = arcMarker.strName
		local width = Apollo.GetTextWidth("CRB_InterfaceSmall",strText)
		width = width+30
		tNewPosition = {}
		tNewPosition.nOL = tMouse.x-(width/2)
		tNewPosition.nOT = tAnchors.nT-28
		tNewPosition.nOR = tNewPosition.nOL+width
		tNewPosition.nOB = tNewPosition.nOT+25
		if tNewPosition.nOR > screenWidth then
			local diff = tNewPosition.nOR - screenWidth
			tNewPosition.nOR = tNewPosition.nOR - diff
			tNewPosition.nOL = tNewPosition.nOL - diff
		end
		if tNewPosition.nOT < 0 then
			tNewPosition.nOT = 0
			tNewPosition.nOB = 25
		end
		if tNewPosition.nOL < 0 then
			local diff = tNewPosition.nOL
			tNewPosition.nOL = tNewPosition.nOL-diff
			tNewPosition.nOR = tNewPosition.nOR-diff
		end
		wndTooltipAltText:SetAnchorOffsets(tNewPosition.nOL, tNewPosition.nOT, tNewPosition.nOR, tNewPosition.nOB)
		wndTooltipAltText:FindChild("WindowText"):SetText(strText)
		wndTooltipAltText:Show(true)
		wndTooltipAltText:ToFront()
	end
	
	return idMarker
end

function Arcompass:OnMouseOverTimer()
	local idMarker = Mouse2Marker(self)
end

function Arcompass:OnMouseOver( wndHandler, wndControl, nLastRelativeMouseX, nLastRelativeMouseY )
	if wndControl == wndHandler then
		local idMarker = Mouse2Marker(self)
		if self.bMouseOverTimerActive ~= true then
			if self.timerMouseOver ~= nil then
				self.timerMouseOver:Start()
			else
				self.timerMouseOver = ApolloTimer.Create(1, true, "OnMouseOverTimer", self)
			end
			self.bMouseOverTimerActive = true
		end
		if wndHandler == wndControl then
			local bLockFrame = self.tToggleOptions.tLockFrame.isToggled
			if bLockFrame == false then
				self.wndMain:FindChild("ResizeLeft"):Show(true)
				self.wndMain:FindChild("ResizeRight"):Show(true)
			end
		end
	end
end

function Arcompass:OnMouseExit( wndHandler, wndControl, x, y )
	if wndControl == wndHandler then
		self.wndMain:FindChild("ResizeLeft"):Show(false)
		self.wndMain:FindChild("ResizeRight"):Show(false)
		if self.timerMouseOver ~= nil then
			self.timerMouseOver:Stop()
			self.wndTooltipAlt:Show(false)
			self.wndTooltipAltText:Show(false)
		end
		self.bMouseOverTimerActive = false
	end
end

function Arcompass:OnOptionsMouseExit( wndHandler, wndControl, x, y )
	if wndControl:GetName() == wndHandler:GetName() then
		self.wndArcompassOptions:Show(false)
		self.bOptionsShown = false
	end
end

function Arcompass:OnMouseClick( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY )
	if eMouseButton == 1 then
		if wndControl ~= self.wndMain then return end
		self:ShowOptionsWindow(wndHandler, wndControl, eMouseButton)
	else
		local idMarker = Mouse2Marker(self)
		if idMarker ~= nil then
			if self.arcMarkers[idMarker] ~= nil then
				if self.arcMarkers[idMarker].eType ~= self.tMarkerTypes.Challenge then
					if self.arcMarkers[idMarker].uOwner ~= nil then
						self.arcMarkers[idMarker].uOwner:ShowHintArrow()
					end
				else
					ChallengesLib.ShowHintArrow(self.arcMarkers[idMarker].uOwner:GetId())
				end
			end
		end
	end
end

function Arcompass:ShowOptionsWindow( wndHandler, wndControl, eMouseButton)
	if self.bOptionsShown then self.bOptionsShown = false
	else self.bOptionsShown = true end
	
	-- Set default anchor points
	local tMouse = Apollo.GetMouse()
	tNewPosition = {}
	tNewPosition.nOL = tMouse.x-(436/2)
	tNewPosition.nOT = tMouse.y+15
	tNewPosition.nOR = tNewPosition.nOL+(436)
	tNewPosition.nOB = tNewPosition.nOT+(544)
	-- Modify if outside the screen
	local screenWidth, screenHeight = Apollo.GetScreenSize()
	if tNewPosition.nOR > screenWidth then
		local diff = tNewPosition.nOR - screenWidth
		tNewPosition.nOR = tNewPosition.nOR - diff
		tNewPosition.nOL = tNewPosition.nOL - diff
	end
	if tNewPosition.nOB > screenHeight then
		tNewPosition.nOT = tMouse.y-544-25
		tNewPosition.nOB = tMouse.y-25
	end
	if tNewPosition.nOL < 0 then
		local diff = tNewPosition.nOL
		tNewPosition.nOL = tNewPosition.nOL-diff+15
		tNewPosition.nOR = tNewPosition.nOR-diff+15
	end
	
	self.wndArcompassOptions:SetAnchorOffsets(tNewPosition.nOL, tNewPosition.nOT, tNewPosition.nOR, tNewPosition.nOB)
	self.wndArcompassOptions:Show(self.bOptionsShown)
	self.wndArcompassOptions:ToFront()
end

function Arcompass:OnOptionsOpacityChanged( wndHandler, wndControl, fNewValue, fOldValue )
	local tMarkerFilter = wndControl:GetData()
	if tMarkerFilter == nil then return end

	tMarkerFilter.opSetting = fNewValue
	if tMarkerFilter.name == "CompassBar" then 
		self.wndMain:FindChild("CompassBackground"):SetOpacity(tMarkerFilter.opSetting*0.01)
	elseif tMarkerFilter.name == "PlayerMarker" then
		self.wndPlayerMarker:SetOpacity(tMarkerFilter.opSetting*0.01)
	else
		self.wndArcompassOptions:FindChild("OptionsBtn"..tMarkerFilter.name):FindChild("SliderBG"):SetText(tostring(math.floor(fNewValue)).."m")
	end
	
	self.bFiltersChanged = true
end

function Arcompass:OnWindowSizeChanged( wndHandler, wndControl )
	self.bWindowResize = true
end

function Arcompass:OnButtonCompassBar( wndHandler, wndControl, eMouseButton )
	if self.nCompassBar == 3 then 
		self.nCompassBar = 1
	else 
		self.nCompassBar = self.nCompassBar + 1 
	end
	local wndCompassBackground = self.wndMain:FindChild("CompassBackground")
	wndCompassBackground:SetSprite(self.eCompassBarTypes[self.nCompassBar])
end

---------------------------------------------------------------------------------------------------
-- ArcompassTooltip Functions
---------------------------------------------------------------------------------------------------

function Arcompass:OnTooltipExit( wndHandler, wndControl, x, y )
	if wndControl:GetName() == wndHandler:GetName() then
		self.wndTooltip:Show(false)
	end
end

-----------------------------------------------------------------------------------------------
-- Arcompass Instance
-----------------------------------------------------------------------------------------------
local ArcompassInst = Arcompass:new()
ArcompassInst:Init()

